from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponseRedirect


@login_required
def merge_objects(request):
    go_back = request.META.get('HTTP_REFERER')
    print("go back: {}".format(go_back))
    if request.method == 'POST':
        keep = request.POST.get("keep", None)
        remove = request.POST.getlist("remove", None)
        model_name = request.POST.get("model_name", None)
        app_name = request.POST.get("app_name", None)
        print('##############################')
        print(keep, remove, model_name, app_name)
        if keep and remove and model_name and app_name:
            print("all good, let's merge")
            try:
                ct = ContentType.objects.get(app_label=app_name, model=model_name).model_class()
            except ObjectDoesNotExist:
                ct = None
                print('no "keep" object found')
                return HttpResponseRedirect(go_back)
            if ct:
                try:
                    keep_obj = ct.objects.filter(pk=keep)[0]
                except IndexError:
                    print("No matching object to keep found")
                    return HttpResponseRedirect(go_back)
                remove_objs = ct.objects.filter(pk__in=remove)
                keep_obj.merge_with(remove_objs)
                return HttpResponseRedirect(go_back)
            else:
                return HttpResponseRedirect(go_back)
        elif remove and model_name and app_name:
            print("all good, let's delete")
            try:
                ct = ContentType.objects.get(app_label=app_name, model=model_name).model_class()
            except ObjectDoesNotExist:
                ct = None
            if ct:
                remove_objs = ct.objects.filter(pk__in=remove)
                if len(remove_objs) > 0:
                    for x in remove_objs:
                        x.delete()
                        print("deleted: {}".format(x))
            return HttpResponseRedirect(go_back)
        else:
            return HttpResponseRedirect(go_back)
    else:
        print('ui, something went wrong')
        return HttpResponseRedirect(go_back)



import json

from dal import autocomplete
from django import http

from apis_core.apis_entities.models import AbstractEntity


class TeiEntAc(autocomplete.Select2ListView):

    def get(self, request, *args, **kwargs):
        page_size = 200
        offset = (int(self.request.GET.get('page', 1))-1)*page_size
        ac_type = self.kwargs['entity']
        if ac_type == "org":
            ac_type = "institution"
        choices = []
        headers = {'Content-Type': 'application/json'}
        ent_model = AbstractEntity.get_entity_class_of_name(ac_type)
        q = self.q.strip()

        res = ent_model.objects.filter(name__startswith=q)
        for r in res[offset:offset+page_size]:
            dates = "time: {} - {}".format(r.start_date, r.end_date)
            f = dict()
            f['id'] = f"{request.build_absolute_uri('/entity/')}{r.pk}"
            if ac_type == 'institution':
                f['type'] = "org"
            else:
                f['type'] = "{}".format(ac_type)
            f['name'] = "{}".format(str(r))
            f['description'] = "{}".format(dates)
            choices.append(f)
        return http.HttpResponse(json.dumps({
                    'item': choices + [],
                    'pagination': {'more': True}
                }), content_type='application/json')


class TeiCompleterAc(autocomplete.Select2ListView):

    def get(self, request, *args, **kwargs):
        page_size = 200
        offset = (int(self.request.GET.get('page', 1))-1)*page_size
        ac_type = self.kwargs['entity']
        if ac_type == "org":
            ac_type = "institution"
        choices = []
        headers = {'Content-Type': 'application/json'}
        ent_model = AbstractEntity.get_entity_class_of_name(ac_type)
        q = self.q.strip()

        res = ent_model.objects.filter(name__startswith=q)
        for r in res[offset:offset+page_size]:
            dates = "time: {} - {}".format(r.start_date, r.end_date)
            f = dict()
            f['tc:value'] = "{}".format(r.uri_set.all()[0])
            if ac_type == 'institution':
                ent_type = "org"
            else:
                ent_type = "{}".format(ac_type)
            f['tc:description'] = f"name: {str(r)}, type: {ent_type}, dates: {dates}".format(dates)
            choices.append(f)
        return http.HttpResponse(json.dumps({
                    'tc:suggestion': choices + [],
                    'pagination': {'more': True}
                }), content_type='application/json')



# Generated by Django 2.2.12 on 2020-06-09 09:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apis_relations', '0002_auto_20200121_1227'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='eventevent',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='eventwork',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='institutionevent',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='institutioninstitution',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='institutionplace',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='institutionwork',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='personevent',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='personinstitution',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='personperson',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='personplace',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='personwork',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='placeevent',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='placeplace',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='placework',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='workwork',
            options={'default_manager_name': 'objects'},
        ),
        migrations.AlterModelManagers(
            name='eventevent',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='eventwork',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='institutionevent',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='institutioninstitution',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='institutionplace',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='institutionwork',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='personevent',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='personinstitution',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='personperson',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='personplace',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='personwork',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='placeevent',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='placeplace',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='placework',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='workwork',
            managers=[
            ],
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-29 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_metainfo', '0003_tempentityclass_published'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='published',
            field=models.BooleanField(default=False),
        ),
    ]



import pandas as pd
from django.db.models import Avg
from django.http import JsonResponse
from django.views.generic import TemplateView

from apis_core.apis_relations.models import *
from .utils import calculate_age


def get_inst_range_data(request):
    df = pd.DataFrame(list(InstitutionInstitution.objects.filter(
        relation_type__name="ist Teil von").distinct().values_list(
        'related_institutionA__name',
        'related_institutionA__start_date__year',
        'related_institutionA__end_date__year',
        'related_institutionB__name'
        )
    ), columns=['name', 'start_year', 'end_year', 'teil von']).fillna(2018)
    df.sort_values('start_year')

    data = {
        "items": "some",
        "title": "{}".format('Kommissionen'),
        "subtitle": "Person Institution relation type {}".format('some type'),
        "legendy": "legendy",
        "legendx": "legendx",
        "categories": "sorted(dates)",
        "measuredObject": "{}".format("Persons average Age"),
        "ymin": 0,
        "x_axis": df['name'].values.tolist(),
        "payload": [
            {
                'name': 'Kommissionen',
                'data': df[['start_year', 'end_year']].values.tolist()
            }
        ]
    }

    return JsonResponse(data, safe=False)


def get_average_members_data(request):
    rel_type = request.GET.get('rel-type')
    rel_inst = request.GET.get('rel-inst')
    start_year = request.GET.get('start-year')
    end_year = request.GET.get('start-year')
    per_inst = PersonInstitution.objects.filter(related_institution_id__in=[3, 2])
    if rel_type:
        data = []
    else:
        start_year = int(str(per_inst.order_by('start_date')[0].start_date)[:4])
        end_year = int(str(per_inst.order_by('-start_date')[0].start_date)[:4])
        qs = [
            {
                'year': x,
                'members_new': per_inst.filter(start_date__year=x).count(),
                'members_all': per_inst.filter(
                    start_date__year__lte=x, end_date__year__gte=x
                ).count(),
            } for x in range(1847, 2015)
        ]
        df = pd.DataFrame(qs)
        payload = [
            {
                'name': 'all members',
                'data': [x for x in df[['members_all']].values.tolist()]
            },
            {
                'name': 'new members',
                'data': [x for x in df[['members_new']].values.tolist()]
            }
        ]
        data = {
            "items": "some",
            "title": "{}".format('Members by Year'),
            "subtitle": "Person Institution relation type {}".format('some type'),
            "legendy": "legendy",
            "legendx": "legendx",
            "categories": "sorted(dates)",
            "measuredObject": "{}".format("Persons average Age"),
            "ymin": 0,
            "x_axis": df['year'].values.tolist(),
            "payload": payload
        }
    return JsonResponse(data, safe=False)


def get_average_age_data(request):
    rel_type = request.GET.get('rel-type')
    rel_inst = request.GET.get('rel-inst')
    start_year = request.GET.get('start-year')
    end_year = request.GET.get('start-year')
    per_inst = PersonInstitution.objects.filter(related_institution_id__in=[3, 2])
    if rel_type:
        data = []
    else:
        start_year = int(str(per_inst.order_by('start_date')[0].start_date)[:4])
        end_year = int(str(per_inst.order_by('-start_date')[0].start_date)[:4])
        qs = [
            {
                'year': x,
                'members_new': per_inst.filter(start_date__year=x).count(),
                'avg_birth_new': per_inst.filter(start_date__year=x)
                .aggregate(
                    Avg('related_person__start_date__year')
                )['related_person__start_date__year__avg'],
                'members_all': per_inst.filter(
                    start_date__year__lte=x, end_date__year__gte=x
                ).count(),
                'avg_birth': per_inst.filter(start_date__year__lte=x, end_date__year__gte=x)
                .aggregate(
                    Avg('related_person__start_date__year')
                )['related_person__start_date__year__avg']
            } for x in range(1847, 2015)
        ]
        df = pd.DataFrame(qs)
        df['avg_age_new'] = df.apply(lambda row: calculate_age(row, 'avg_birth_new'), axis=1)
        df['avg_age_all'] = df.apply(lambda row: calculate_age(row, 'avg_birth'), axis=1)
        payload = [
            {
                'name': 'average age all',
                'data': [x for x in df[['avg_age_all']].values.tolist()]
            },
            {
                'name': 'average age new',
                'data': [x for x in df[['avg_age_new']].values.tolist()]
            }
        ]
        data = {
            "items": "some",
            "title": "{}".format('Average Age by Year'),
            "subtitle": "Person Institution relation type {}".format('some type'),
            "legendy": "legendy",
            "legendx": "legendx",
            "categories": "sorted(dates)",
            "measuredObject": "{}".format("Persons average Age"),
            "ymin": 0,
            "x_axis": df['year'].values.tolist(),
            "payload": payload
        }
    return JsonResponse(data, safe=False)


def get_heatmap_data(request):

    rel_type = request.GET.get('rel-type')
    places = [
        (
            x.related_place.lat,
            x.related_place.lng,
        ) for x in PersonPlace.objects.filter(
            relation_type__name=rel_type
        ).exclude(related_place__lat__isnull=True)
    ]
    return JsonResponse(places, safe=False)


class HeatMapView(TemplateView):
    template_name = "apis_vis/heatmap.html"


class AvgAge(TemplateView):
    template_name = "apis_vis/avgage.html"


class MembersAmountPerYear(TemplateView):
    template_name = "apis_vis/avgmemperyear.html"


class InstRange(TemplateView):
    template_name = "apis_vis/inst_range.html"



import numpy as np
from keras.models import load_model
from keras.preprocessing.text import Tokenizer
import pickle
from keras import backend as K
import os

import spacy
from spacy.matcher import Matcher

import de_core_news_sm

from apis_core.vocabularies.models import VocabsBaseClass

nlp = de_core_news_sm.load()

lst_orth = []
lst_orth_dict = dict()


def extract_verbs_from_entity(
        ent, lst_orth, lst_orth_dict, accept_pos=['VERB', 'AUX', 'NOUN', 'ADP', 'PART'], add=True):
    res = []
    for y in ent:
        head = y
        tokens = []
        lemmas = []
        pos_tags = []
        shapes = []
        while head:
            if head.head != head:
                head = head.head
                if head.pos_ in accept_pos:
                    if head.orth in lst_orth:
                        tokens.append(lst_orth_dict[head.orth])
                    elif add:
                        lst_orth.append(head.orth)
                        lst_orth_dict[head.orth] = len(lst_orth)
                        tokens.append(len(lst_orth))
                    else:
                        continue
                    pos_tags.append(head.pos)
                    shapes.append(head.shape)
            else:
                head = False
    return tokens, lemmas, pos_tags, shapes


def test_model(model, sent):
    K.clear_session()
    script_dir = os.path.dirname(os.path.realpath('__file__'))
    fileh = open(os.path.join(script_dir,
                              'data/nlp_models/{}_vocab.obj'.format(model)), 'rb')
    lst_orth, lst_orth_dict, lst_labels, lst_labels_dict, lst_zero_label, lst_labels_dhae2 = pickle.load(fileh)
    model = load_model(os.path.join(script_dir,
                       'data/nlp_models/{}.h5'.format(model)))
    result = []
    txt = nlp(sent)
    tokens_lst = []
    for ent in txt.ents:
        print(ent)
        tokens, lemmas, pos_tags, shapes = extract_verbs_from_entity(ent, lst_orth, lst_orth_dict, add=False)
        if len(tokens) > 0:
            tokens_lst.append(tokens)
    x_matrix2 = np.array(tokens_lst)
    print(x_matrix2)
    tokenizer = Tokenizer(num_words=len(lst_orth))
    x_matrix3 = tokenizer.sequences_to_matrix(tokens_lst, mode='binary')
    zz = model.predict(x_matrix3, batch_size=32, verbose=1)
    for idx1, z in enumerate(zz):
        for idx, x in enumerate(zz[idx1]):
            v_id = '-'
            for k in lst_labels_dict.keys():
                if lst_labels_dict[k] == idx:
                    v_id = VocabsBaseClass.objects.get(id=k).name
            result.append((str(txt.ents[idx1]), idx, v_id, x))
    return result



# -*- coding: utf-8 -*-
from django.conf.urls import url

from . import views

app_name = 'apis_labels'

urlpatterns = [
url(r'^list/$', views.LabelListView.as_view(), name='label_list'),
url(r'^create/$', views.label_create, name='label_create'),
url(r'^edit/(?P<pk>[0-9]+)$', views.label_edit, name='label_edit'),
url(r'^delete/(?P<pk>[0-9]+)$', views.LabelDelete.as_view(), name='label_delete'),
]



# -*- coding: utf-8 -*-
from crispy_forms.bootstrap import Accordion, AccordionGroup
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset
from crispy_forms.layout import Submit
from dal import autocomplete
from django import forms
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.validators import URLValidator
from django.db.models.fields import BLANK_CHOICE_DASH
from django.forms import ModelMultipleChoiceField, ModelChoiceField
from django.urls import reverse

from apis_core.apis_metainfo.models import Text, Uri, Collection
from apis_core.apis_vocabularies.models import TextType
from apis_core.helper_functions import DateParser
from apis_core.helper_functions.RDFParser import RDFParser
from .fields import ListSelect2, Select2Multiple
from .models import AbstractEntity

if "apis_highlighter" in settings.INSTALLED_APPS:
    from apis_highlighter.models import AnnotationProject


class SearchForm(forms.Form):
    search = forms.CharField(label="Search")

    @property
    def helper(self):
        helper = FormHelper()
        helper.form_id = "searchForm"
        helper.form_tag = False
        helper.add_input(Submit("fieldn", "search"))
        helper.form_method = "GET"
        return helper


def get_entities_form(entity):

    # TODO __sresch__ : consider moving this class outside of the function call to avoid redundant class definitions
    class GenericEntitiesForm(forms.ModelForm):
        class Meta:
            model = AbstractEntity.get_entity_class_of_name(entity)

            exclude = [
                "start_date",
                "start_start_date",
                "start_end_date",
                "start_date_is_exact",
                "end_date",
                "end_start_date",
                "end_end_date",
                "end_date_is_exact",
                "text",
                "source",
                "published",
            ]
            exclude.extend(model.get_related_entity_field_names())
            exclude.extend(model.get_related_relationtype_field_names())

        def __init__(self, *args, **kwargs):
            super(GenericEntitiesForm, self).__init__(*args, **kwargs)
            self.helper = FormHelper()
            self.helper.form_class = entity.title() + "Form"
            self.helper.form_tag = False
            self.helper.help_text_inline = True
            acc_grp1 = Fieldset("Metadata {}".format(entity.title()))
            acc_grp2 = AccordionGroup("MetaInfo", "references", "notes", "review")
            attrs = {
                "data-placeholder": "Type to get suggestions",
                "data-minimum-input-length": getattr(settings, "APIS_MIN_CHAR", 3),
                "data-html": True,
            }

            # list to catch all fields that will not be inserted into accordion group acc_grp2
            fields_list_unsorted = []

            for f in self.fields.keys():
                if isinstance(
                    self.fields[f], (ModelMultipleChoiceField, ModelChoiceField)
                ):
                    v_name_p = str(self.fields[f].queryset.model.__name__)
                    if isinstance(self.fields[f], ModelMultipleChoiceField):
                        widget1 = Select2Multiple
                    else:
                        widget1 = ListSelect2
                    if (
                        ContentType.objects.get(
                            app_label__in=[
                                "apis_entities",
                                "apis_metainfo",
                                "apis_relations",
                                "apis_vocabularies",
                                "apis_labels",
                            ],
                            model=v_name_p.lower(),
                        ).app_label.lower()
                        == "apis_vocabularies"
                    ):
                        self.fields[f].widget = widget1(
                            url=reverse(
                                "apis:apis_vocabularies:generic_vocabularies_autocomplete",
                                kwargs={"vocab": v_name_p.lower(), "direct": "normal"},
                            ),
                            attrs=attrs,
                        )
                        if self.instance:
                            res = []
                            if isinstance(self.fields[f], ModelMultipleChoiceField):
                                try:
                                    for x in getattr(self.instance, f).all():
                                        res.append((x.pk, x.label))
                                except ValueError:
                                    pass
                                self.fields[f].initial = res
                                self.fields[f].choices = res
                            else:
                                try:
                                    res = getattr(self.instance, f)
                                    if res is not None:
                                        self.fields[f].initial = (res.pk, res.label)
                                        self.fields[f].choices = [
                                            (res.pk, res.label),
                                        ]
                                except ValueError:
                                    res = ""
                if f not in acc_grp2:
                    # append to unsorted list, so that it can be sorted and afterwards attached to accordion group acc_grp1
                    fields_list_unsorted.append(f)

            def sort_fields_list(list_unsorted, entity_label):
                """
                Sorts a list of model fields according to a defined order.


                :param list_unsorted: list
                    The unsorted list of fields.

                :param entity_label: str
                    The string representation of entity type, necessary to find the entity-specific ordering (if it is defined)


                :return: list
                    The sorted list if entity-specific ordering was defined, the same unordered list if not.
                """

                entity_settings = getattr(settings, 'APIS_ENTITIES', None)

                if entity_settings is None:
                    return list_unsorted

                sort_preferences = entity_settings[entity_label].get('form_order', None)
                sort_preferences_used = []

                if sort_preferences is None:
                    return list_unsorted
                else:
                    # list of tuples to be sorted later
                    field_rank_pair_list = []
                    for field in list_unsorted:
                        if field in sort_preferences:
                            # if this succeeds, then the field has been given a priorites ordering above
                            ranking_by_index = sort_preferences.index(field)
                            sort_preferences_used.append(field)
                            field_rank_pair = (field, ranking_by_index)
                        else:
                            # if no ordering for the field was found, then give it 'Inf'
                            # so that it will be attached at the end.
                            field_rank_pair = (field, float('Inf'))
                        field_rank_pair_list.append(field_rank_pair)
                    # Make a check if all items of sort_preferences were used. If not, this indicates an out of sync setting
                    # if len(sort_preferences) > 0:
                    if len(sort_preferences_used) != len(sort_preferences):

                        differences = []
                        for p in sort_preferences_used:
                            if p not in sort_preferences:
                                differences.append(p)
                        for p in sort_preferences:
                            if p not in sort_preferences_used:
                                differences.append(p)

                        raise Exception(
                            "An item of the entity setting 'form_order' list was not used. \n"
                            "This propably indicates that the 'form_order' settings is out of sync with the effective django models.\n"
                            f"The relevant entity is: {entity_label}\n"
                            f"And the differences between used list and settings list are: {differences}"
                        )
                    # sort the list according to the second element in each tuple
                    # and then take the first elements from it and return as list
                    return [ t[0] for t in sorted(field_rank_pair_list, key=lambda x: x[1]) ]

            # sort field list, iterate over it and append each element to the accordion group
            for f in sort_fields_list(fields_list_unsorted, entity):
                acc_grp1.append(f)

            self.helper.layout = Layout(Accordion(acc_grp1, acc_grp2))
            self.fields["status"].required = False
            self.fields["collection"].required = False
            self.fields["start_date_written"].required = False
            self.fields["end_date_written"].required = False

            instance = getattr(self, "instance", None)
            if instance != None:

                if instance.start_date_written:
                    self.fields[
                        "start_date_written"
                    ].help_text = DateParser.get_date_help_text_from_dates(
                        single_date=instance.start_date,
                        single_start_date=instance.start_start_date,
                        single_end_date=instance.start_end_date,
                        single_date_written=instance.start_date_written,
                    )
                else:
                    self.fields[
                        "start_date_written"
                    ].help_text = DateParser.get_date_help_text_default()

                if instance.end_date_written:
                    self.fields[
                        "end_date_written"
                    ].help_text = DateParser.get_date_help_text_from_dates(
                        single_date=instance.end_date,
                        single_start_date=instance.end_start_date,
                        single_end_date=instance.end_end_date,
                        single_date_written=instance.end_date_written,
                    )
                else:
                    self.fields[
                        "end_date_written"
                    ].help_text = DateParser.get_date_help_text_default()

        def save(self, *args, **kwargs):
            obj = super(GenericEntitiesForm, self).save(*args, **kwargs)
            if obj.collection.all().count() == 0:
                col_name = getattr(
                    settings, "APIS_DEFAULT_COLLECTION", "manually created entity"
                )
                col, created = Collection.objects.get_or_create(name=col_name)
                obj.collection.add(col)
            return obj

    return GenericEntitiesForm


class GenericEntitiesStanbolForm(forms.Form):
    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        entity = RDFParser(cd["entity"], self.entity.title()).get_or_create()
        return entity

    def __init__(self, entity, *args, **kwargs):

        attrs = {
            "data-placeholder": "Type to get suggestions",
            "data-minimum-input-length": getattr(settings, "APIS_MIN_CHAR", 3),
            "data-html": True,
            "style": "width: auto",
        }
        ent_merge_pk = kwargs.pop("ent_merge_pk", False)
        super(GenericEntitiesStanbolForm, self).__init__(*args, **kwargs)
        self.entity = entity
        self.helper = FormHelper()
        form_kwargs = {"entity": entity}
        url = reverse(
            "apis:apis_entities:generic_entities_autocomplete",
            args=[entity.title(), "remove"],
        )
        label = "Create {} from reference resources".format(entity.title())
        button_label = "Create"
        if ent_merge_pk:
            form_kwargs["ent_merge_pk"] = ent_merge_pk
            url = reverse(
                "apis:apis_entities:generic_entities_autocomplete",
                args=[entity.title(), ent_merge_pk],
            )
            label = "Search for {0} in reference resources or db".format(entity.title())
            button_label = "Merge"
        self.helper.form_action = reverse(
            "apis:apis_entities:generic_entities_stanbol_create", kwargs=form_kwargs
        )
        self.helper.add_input(Submit("submit", button_label))
        self.fields["entity"] = autocomplete.Select2ListCreateChoiceField(
            label=label,
            widget=ListSelect2(url=url, attrs=attrs),
            validators=[URLValidator],
        )


class FullTextForm(forms.Form):
    def save(self, entity):
        cd = self.cleaned_data
        text = None
        for f in cd.keys():
            text_type = TextType.objects.get(pk=f.split("_")[1])
            text = Text.objects.filter(tempentityclass=entity, kind=text_type)
            if text.count() == 1:
                text = text[0]
                text.text = cd[f]
                text.save()
            elif text.count() == 0:
                text = Text(text=cd[f], kind=text_type)
                text.save()
                entity.text.add(text)
        return text

    def __init__(self, *args, **kwargs):
        if "entity" in kwargs.keys():
            entity = kwargs.pop("entity", None)
        else:
            entity = None
        if "instance" in kwargs.keys():
            instance = kwargs.pop("instance", None)
        else:
            instance = None
        super(FullTextForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = "FullTextForm"
        self.helper.form_tag = False
        self.helper.help_text_inline = True
        collections = []
        if instance:
            for i in instance.collection.all():
                collections.append(i)
        try:
            if len(collections) > 0:
                q = TextType.objects.filter(
                    entity__iexact=entity, collections__in=collections
                )
            else:
                q = TextType.objects.filter(entity__iexact=entity)
            for txt in q:
                self.fields["text_" + str(txt.pk)] = forms.CharField(
                    label=txt.name,
                    help_text=txt.description,
                    required=False,
                    widget=forms.Textarea,
                )
            if instance:
                for t in instance.text.all():
                    if "text_" + str(t.kind.pk) in self.fields.keys():
                        self.fields["text_" + str(t.kind.pk)].initial = t.text
        except:
            pass


class PersonResolveUriForm(forms.Form):
    # person = forms.CharField(label=False, widget=al.TextWidget('PersonAutocomplete'))
    person = forms.CharField(label=False)
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if cd["person"].startswith("http"):
            uri = Uri.objects.create(uri=cd["person"], entity=site_instance)
        else:
            uri = Uri.objects.create(uri=cd["person_uri"], entity=site_instance)
        return uri

    def __init__(self, *args, **kwargs):
        entity_type = kwargs.pop("entity_type", False)
        self.request = kwargs.pop("request", False)
        super(PersonResolveUriForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False

    def clean(self):
        cleaned_data = super(PersonResolveUriForm, self).clean()
        if Uri.objects.filter(uri=cleaned_data["person_uri"]).exists():
            self.add_error("person", "This Person has already been added to the DB.")
        elif cleaned_data["person"].startswith("http"):
            if Uri.objects.filter(uri=cleaned_data["person"]).exists():
                self.add_error("person", "This URI has already been added to the DB.")


class NetworkVizFilterForm(forms.Form):
    ann_include_all = forms.BooleanField(
        required=False,
        label="Include general relations",
        help_text="""Not all relations are connected to an annotation.\
        If checked relations that are not attached to an annotation are include.\
        This setting is only used when an Annotation project is specified.""",
    )
    start_date = forms.CharField(
        label="Start date",
        required=False,
        widget=forms.TextInput(
            attrs={"data-provide": "datepicker", "data-date-format": "dd.mm.yyyy"}
        ),
    )
    end_date = forms.CharField(
        label="End date",
        required=False,
        widget=forms.TextInput(
            attrs={"data-provide": "datepicker", "data-date-format": "dd.mm.yyyy"}
        ),
    )

    def __init__(self, *args, **kwargs):
        rel_attrs = {
            "data-placeholder": "Type to get suggestions",
            "data-minimum-input-length": getattr(settings, "APIS_MIN_CHAR", 3),
            "data-html": True,
        }
        attrs = {
            "data-placeholder": "Type to get suggestions",
            "data-minimum-input-length": getattr(settings, "APIS_MIN_CHAR", 3),
            "data-html": True,
        }
        super(NetworkVizFilterForm, self).__init__(*args, **kwargs)
        self.fields["select_relation"] = forms.ChoiceField(
            label="Relation type",
            choices=list(
                ("-".join(x.name.split()), x.name)
                for x in ContentType.objects.filter(app_label="apis_relations")
            ),
            help_text="Include only relations related to this annotation project \
            (See the include general relations checkbox)",
        )
        self.fields["select_relation"].initial = ("person-place", "person place")
        self.fields["search_source"] = autocomplete.Select2ListCreateChoiceField(
            label="Search source",
            widget=ListSelect2(
                url=reverse(
                    "apis:apis_entities:generic_network_entities_autocomplete",
                    kwargs={"entity": "person"},
                ),
                attrs=attrs,
            ),
        )
        self.fields["search_target"] = autocomplete.Select2ListCreateChoiceField(
            label="Search target",
            widget=ListSelect2(
                url=reverse(
                    "apis:apis_entities:generic_network_entities_autocomplete",
                    kwargs={"entity": "place"},
                ),
                attrs=attrs,
            ),
        )
        self.fields["select_kind"] = autocomplete.Select2ListCreateChoiceField(
            label="Select kind",
            widget=ListSelect2(
                url=reverse(
                    "apis:apis_vocabularies:generic_vocabularies_autocomplete",
                    kwargs={"vocab": "personplacerelation", "direct": "normal"},
                ),
                attrs=rel_attrs,
            ),
        )
        if "apis_highlighter" in settings.INSTALLED_APPS:
            self.fields["annotation_proj"] = forms.ChoiceField(
                label="Annotation Project",
                choices=BLANK_CHOICE_DASH
                + list((x.pk, x.name) for x in AnnotationProject.objects.all()),
                required=False,
                help_text="Include only relations related to this annotation project \
                (See the include general relations checkbox)",
            )
        self.helper = FormHelper()
        self.helper.form_class = "FilterNodesForm"
        self.helper.form_action = reverse("apis:apis_core:NetJson-list")
        self.helper.add_input(Submit("Submit", "Add nodes"))
        self.order_fields(
            (
                "select_relation",
                "ann_include_all",
                "annotation_proj",
                "search_source",
                "select_kind",
                "search_target",
            )
        )


class GenericFilterFormHelper(FormHelper):
    def __init__(self, *args, **kwargs):
        super(GenericFilterFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = "genericFilterForm"
        self.form_method = "GET"
        self.add_input(Submit("Filter", "Filter"))




from django.conf.urls import url

from . import views

app_name = 'apis_metainfo'

urlpatterns = [
    url(
        r'^apis/metainfo/uri/$',
        views.UriListView.as_view(),
        name='uri_browse'
    ),
    url(
        r'^uri/detail/(?P<pk>[0-9]+)$',
        views.UriDetailView.as_view(),
        name='uri_detail'
    ),
    url(
        r'^uri/create/$',
        views.UriCreate.as_view(),
        name='uri_create'
    ),
    url(
        r'^uri/edit/(?P<pk>[0-9]+)$',
        views.UriUpdate.as_view(),
        name='uri_edit'
    ),
    url(
        r'^uri/delete/(?P<pk>[0-9]+)$',
        views.UriDelete.as_view(),
        name='uri_delete'),
]



# Generated by Django 2.2.13 on 2020-07-22 12:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apis_metainfo', '0005_text_lang'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='text',
            name='lang',
        ),
    ]



# Generated by Django 2.1.12 on 2020-01-21 12:27

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BrowsConf',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model_name', models.CharField(blank=True, help_text='The name of the model class you like to analyse.', max_length=255)),
                ('label', models.CharField(blank=True, help_text='The label of the value of interest.', max_length=255)),
                ('field_path', models.CharField(blank=True, help_text='The constructor of to the value of interest.', max_length=255)),
            ],
        ),
    ]



import requests


def create_qs(row, cell, q_field="dateOfBirth", char_limit=0):
    """turns the value of the cell into a lobid-querystring"""
    try:
        if len(row[cell]) > char_limit:
            qs = "{}:{} ".format(q_field, row[cell])
        else:
            qs = "{}:{}*".format(q_field, row[cell])
    except:
        qs = ""
    return qs


def lobid_qs(row, q_field='name', add_fields=[], base_url="https://lobid.org/gnd/search?q="):
    """ creates a lobid query string from the passed in fields"""
    search_url = base_url+row[q_field]+"&filter=type:Person"
    if add_fields:
        filters = []
        for x in add_fields:
            if x:
                filters.append(row[x])
        search_url = "{} AND {}".format(search_url, "AND ".join(filters))
    return search_url


def search_lobid(row, qs_field='query'):
    """ sends the value of the passed in field to lobid and returns the results in a dict """
    query = row[qs_field]
    result = {
        'query': query,
        'status': 0,
        'error': "",
        'hits': 0,
        'gnd': []
    }
    r = requests.get(query)
    try:
        r = requests.get(query)
    except requests.ConnectionError:
        result['error'] = "Connection Error"
        return result
    if r:
        if r.status_code == 200:
            result['status'] = r.status_code
            result['hits'] = r.json()['totalItems']
            if result['hits'] == 0:
                gnd = []
                print('zero hits')
            elif result['hits'] == 1:
                print("one hit")
                result['gnd'] = [r.json()['member'][0]['gndIdentifier']]
            else:
                print("{} hits".format(result['hits']))
                result['gnd'] = [x['gndIdentifier'] for x in r.json()['member']]
        else:
            result['status'] = r.status_code
    return result


# do something like below to enrich a csv with lobid

# import pandas as pd
# from pylobid import *
#
# file = "dump_from_gsheet/ASBW_persName.csv"
# df = pd.read_csv(file)
# df['dateOfBirth'] = df.apply(
#     lambda row: create_qs(row, 'Geburtsdatum', q_field="dateOfBirth", char_limit=4),
#     axis=1
# )
# df['dateOfDeath'] = df.apply(
#     lambda row: create_qs(row, 'Todesdatum', q_field="dateOfDeath", char_limit=4),
#     axis=1
# )
# df['forename'] = df.apply(
#     lambda row: create_qs(
#         row, 'Vorname', q_field="preferredNameEntityForThePerson.forename", char_limit=0
#     ),
#     axis=1
# )
# df['surname'] = df.apply(
#     lambda row: create_qs(
#         row, 'Nachname', q_field="preferredNameEntityForThePerson.surname", char_limit=0
#     ),
#     axis=1
# )
# df['query'] = df.apply(
#     lambda row: lobid_qs(row, add_fields=['forename', 'dateOfBirth', 'dateOfDeath']),
#     axis=1
# )



# Generated by Django 2.1.12 on 2020-01-21 12:27

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_entities', '0001_initial'),
        ('apis_relations', '0001_initial'),
        ('apis_metainfo', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='VocabNames',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='VocabsBaseClass',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('description', models.TextField(blank=True, help_text='Brief description of the used term.')),
                ('status', models.CharField(choices=[('rej', 'rejected'), ('ac', 'accepted'), ('can', 'candidate'), ('del', 'deleted')], default='can', max_length=4)),
            ],
        ),
        migrations.CreateModel(
            name='VocabsUri',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uri', models.URLField()),
                ('domain', models.CharField(blank=True, max_length=255)),
                ('rdf_link', models.URLField(blank=True)),
                ('loaded', models.BooleanField(default=False)),
                ('loaded_time', models.DateTimeField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='CollectionType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='EventType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='LabelType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='PlaceType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='ProfessionType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='RelationBaseClass',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
                ('name_reverse', models.CharField(blank=True, help_text='Inverse relation like: "is sub-class of" vs. "is super-class of".', max_length=255, verbose_name='Name reverse')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='TextType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
                ('entity', models.CharField(max_length=255)),
                ('collections', models.ManyToManyField(blank=True, to='apis_metainfo.Collection')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='Title',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
                ('abbreviation', models.CharField(blank=True, max_length=10)),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='WorkType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.AddField(
            model_name='vocabsuri',
            name='vocab',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_vocabularies.VocabsBaseClass'),
        ),
        migrations.AddField(
            model_name='vocabsbaseclass',
            name='parent_class',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_vocabularies.VocabsBaseClass'),
        ),
        migrations.AddField(
            model_name='vocabsbaseclass',
            name='userAdded',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='vocabsbaseclass',
            name='vocab_name',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.VocabNames'),
        ),
        migrations.CreateModel(
            name='EventEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
                ('eventB_set', models.ManyToManyField(blank=True, related_name='eventA_relationtype_set', through='apis_relations.EventEvent', to='apis_entities.Event')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='EventWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionInstitutionRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
                ('institutionB_set', models.ManyToManyField(blank=True, related_name='institutionA_relationtype_set', through='apis_relations.InstitutionInstitution', to='apis_entities.Institution')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionPlaceRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonInstitutionRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonPersonRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
                ('personB_set', models.ManyToManyField(blank=True, related_name='personA_relationtype_set', through='apis_relations.PersonPerson', to='apis_entities.Person')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonPlaceRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PlaceEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PlacePlaceRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
                ('placeB_set', models.ManyToManyField(blank=True, related_name='placeA_relationtype_set', through='apis_relations.PlacePlace', to='apis_entities.Place')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PlaceWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='WorkWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
                ('workB_set', models.ManyToManyField(blank=True, related_name='workA_relationtype_set', through='apis_relations.WorkWork', to='apis_entities.Work')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_vocabularies.relationbaseclass',),
        ),
    ]



import re
from collections import OrderedDict

from rest_framework import renderers


class NetJsonRenderer(renderers.JSONRenderer):
    media_type = "application/json"
    format = "json+net"

    def render(self, data, media_type=None, renderer_context=None):
        try:
            test = re.search(r"/relations/([^/]+)/", data["results"][0]["url"])
        except IndexError:
            return super().render(
                data, accepted_media_type=media_type, renderer_context=renderer_context
            )
        if test:
            rel = test.group(1)
            for r in data["results"][0].keys():
                if r.startswith("related_"):
                    r2 = r.split("_")[1]
                    rel2 = re.match("^{}[a-z]*".format(r2), rel)
                    if rel2:
                        source = r
                    elif r.endswith('A'):
                        source = r
                    elif r.endswith('B'):
                        target = r
                    rel2 = re.match("^[a-z]*?{}$".format(r2), rel)
                    if rel2:
                        target = r
            results2 = []
            for d in data["results"]:
                d2 = OrderedDict(
                    [
                        ("target", v)
                        if k == target
                        else ("source", v)
                        if k == source
                        else (k, v)
                        for k, v in d.items()
                    ]
                )
                results2.append(d2)
            data["results"] = results2
            res3 = super().render(data, accepted_media_type=media_type, renderer_context=renderer_context)
            return res3



from django.contrib import admin
from csvexport.actions import csvexport


from .models import Label


# Register your models here.
class BaseAdmin(admin.ModelAdmin):
    
    search_fields = ('label',)
    list_filter = (
        "isoCode_639_3",
        "label_type",
    )
    list_display = ('label', 'temp_entity', )
    exclude = ('temp_entity', )
    actions = [csvexport]

admin.site.register(Label, BaseAdmin)


import importlib
import inspect
from django.contrib.contenttypes.models import ContentType
from django.db.models.base import ModelBase


class GetContentTypes:


    # class memeber dictionary, used for caching ContentType objects in method get_content_type_of_class_or_instance
    class_content_type_dict = {}


    def get_names(self):
        """Function to create Module/Class name tuples

        Returns:
            [list]: [list of Modile/Class tuples]
        """            
        res = []
        for cls in self._lst_cont:
            res.append((cls.__module__.split('.')[-2], cls.__name__))
        return res

    def get_model_classes(self):
        """Returns list of django model classes

        Returns:
            [list]: [django model classes]
        """            
        return self._lst_cont


    @classmethod
    def get_content_type_of_class_or_instance(cls, model_class_or_instance):
        """
        Helper method which caches ContentType of a given model class or instance. When first called on a model
        it fetches its respective ContentType from DB and caches it in a class dictionary. Later this dictionary is
        called

        :param model_class_or_instance: can be either a model class or an instance of a model class
        :return: The django ContentType object for the given parameter
        """

        model_class = None
        if type(model_class_or_instance) is ModelBase:
            # true if model_class_or_instance is model class
            model_class = model_class_or_instance
        elif type(type(model_class_or_instance)) is ModelBase:
            # true if model_class_or_instance is model instance
            model_class = type(model_class_or_instance)
        else:
            raise Exception(
                f"Passed argument is neither model class nor model instance: {type(model_class_or_instance)}"
            )

        if model_class not in cls.class_content_type_dict:
            cls.class_content_type_dict[model_class] = ContentType.objects.get(model=model_class.__name__)
        return cls.class_content_type_dict[model_class]


    def __init__(self, lst_conts=None):
        """

        Args:
            lst_conts ([list], optional): [list of entity names]. Defaults to list of apis_core entities.
        """        
        models_exclude = ["texttype_collections", "relationbaseclass", "baserelationmanager", "relationpublishedqueryset"]
        apis_modules = ['apis_core.apis_metainfo.models', 'apis_core.apis_vocabularies.models', 'apis_core.apis_entities.models', 'apis_core.apis_relations.models']
        if lst_conts is not None:
            r2 = []
            for c in lst_conts:
                for c2 in apis_modules:
                    if c in c2:
                        r2.append(c2)
            apis_modules = r2
        lst_cont_pre = [importlib.import_module(x) for x in apis_modules]
        lst_cont = []
        for m in lst_cont_pre:
            for cls_n in dir(m):
                if not cls_n.startswith('__') and "__module__" in list(dir(getattr(m, cls_n))):
                    if getattr(m, cls_n).__module__ in apis_modules and cls_n.lower() not in models_exclude and not "abstract" in cls_n.lower() and inspect.isclass(getattr(m, cls_n)):
                        lst_cont.append(getattr(m, cls_n))
        self._lst_cont = lst_cont


from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db.models.query import QuerySet

from apis_core.apis_metainfo.models import Text, TempEntityClass
from apis_highlighter.models import Annotation

if 'annotator agreement' in getattr(settings, "APIS_COMPONENTS", []):
    from nltk.metrics import AnnotationTask
    import pandas as pd
    from sklearn.metrics import precision_recall_fscore_support
    import numpy as np


class InternalDataAgreement(object):
    @staticmethod
    def internal_data_agreement_calc(texts, ann1, ann2, anno_proj=None, format_string='start_end_text'):
        if type(texts) == QuerySet:
            q = {'text__in': texts}
        else:
            q = {'text': texts}
        lst_user_ann = []
        if anno_proj:
            q['annotation_project_id'] = anno_proj
        lst_ann_fin = dict()
        for v in [ann1, ann2]:
            lst_ann = []
            q['user_added_id'] = v
            for an in Annotation.objects.filter(**q).order_by('start'):
                lst_ann.append(an.annotation_hash(format_string=format_string))
                if an.user_added_id not in lst_user_ann:
                    lst_user_ann.append(an.user_added_id)
            lst_ann_fin[v] = lst_ann
        ann_task = []
        for ann in lst_ann_fin[ann1]:
            ann_task.append((ann1, ann, 1))
            if ann in lst_ann_fin[ann2]:
                ann_task.append((ann2, ann, 1))
            else:
                ann_task.append((ann2, ann, 0))
        for ann in lst_ann_fin[ann2]:
            if ann not in lst_ann_fin[ann1]:
                ann_task.append((ann2, ann, 1))
                ann_task.append((ann1, ann, 0))

        if len(ann_task) == 0:
            return None
        else:
            return AnnotationTask(data=ann_task)

    def precision_recall_calc(self, texts, gold_standard, user_group, anno_proj=None, format_string='start_end_text'):
        if type(texts) == QuerySet:
            q = {'text__in': texts}
        else:
            q = {'text': texts}
        if anno_proj:
            q['annotation_project_id'] = anno_proj
        lst_ann_fin = dict()
        if not format_string:
            format_string = 'start_end_text'
        user_list = [gold_standard]
        user_list.extend(User.objects.filter(groups__pk=user_group).exclude(pk=gold_standard)
                         .values_list('pk', flat=True))
        df = pd.DataFrame()
        for v in user_list:
            lst_ann = []
            q['user_added_id'] = v
            for an in Annotation.objects.filter(**q).order_by('start'):
                lst_ann.append(an.annotation_hash(format_string=format_string))
            lst_ann_fin[v] = lst_ann
        gold_username = User.objects.get(pk=gold_standard).username
        for u in lst_ann_fin.keys():
            ann_username = User.objects.get(pk=u).username
            if u == gold_standard:
                continue
            gold_list = []
            ann_list = []
            for ann in lst_ann_fin[u]:
                if ann in lst_ann_fin[gold_standard]:
                    gold_list.append(1)
                    ann_list.append(1)
                else:
                    gold_list.append(0)
                    ann_list.append(1)
            for ann in lst_ann_fin[gold_standard]:
                if ann not in lst_ann_fin[u]:
                    gold_list.append(1)
                    ann_list.append(0)
            prec_res = precision_recall_fscore_support(np.array(gold_list), np.array(ann_list), average='binary',
                                                       pos_label=1)
            for idx, k in enumerate(['precission', 'recall', 'fbeta_score', 'support']):
                df.loc[ann_username, k] = prec_res[idx]
        return df

    def get_html_table(self):
        css_class = 'table table-bordered table-hover'
        self.html_tables = dict()
        self.html_tables_gold = None
        if isinstance(self.texts, dict):
            for txt_id in self.texts.keys():
                self.html_tables[txt_id] = self.texts[txt_id].to_html(classes=css_class)
            if self.texts_gold is not None:
                self.html_tables_gold = dict()
                for txt_id in self.texts_gold.keys():
                    self.html_tables_gold[txt_id] = self.texts_gold[txt_id].to_html(classes=css_class)
        elif isinstance(self.texts, pd.DataFrame):
            self.html_tables = self.texts.to_html(classes=css_class)
            if self.texts_gold is not None:
                self.html_tables_gold = self.texts_gold.to_html(classes=css_class)
        return self.html_tables, self.html_tables_gold

    def __init__(self, texts, anno_proj, user_group, metrics='Do_alpha',
                 format_string='start_end_text', combine=False, gold_standard=False):
        if not type(texts) == QuerySet:
            m_name = ContentType.objects.get_for_model(texts).name
            if m_name == 'person' or m_name == 'place' or m_name == 'institution' or m_name == 'event' or m_name == 'work':
                texts = Text.objects.filter(tempentityclass=texts).distinct()
            elif m_name == 'collection':
                t = TempEntityClass.objects.filter(collection=texts)
                texts = Text.objects.filter(tempentityclass__in=t).distinct()
        self.texts_gold = None
        self._list_users = dict()
        if combine:
            self.texts = pd.DataFrame()
        else:
            self.texts = dict()
        if combine:
            texts = [texts]
        if gold_standard:
            self.texts_gold = dict()
        if not user_group:
            user_qs = dict()
        else:
            user_qs = {'groups__pk': user_group}
        for txt in texts:
            df = pd.DataFrame()
            test = False
            for ann1 in User.objects.filter(**user_qs):
                for ann2 in User.objects.filter(**user_qs):
                    if ann1 == ann2:
                        t = None
                        continue
                    try:
                        t = getattr(
                            self.internal_data_agreement_calc(
                                txt,
                                ann1.pk,
                                ann2.pk,
                                anno_proj=anno_proj,
                                format_string=format_string),metrics)()
                        if pd.notnull(t):
                            test = True
                    except AttributeError as ex:
                        t = None
                    df.loc[ann1.username, ann2.username] = t
                    if ann1.pk not in self._list_users.keys():
                        self._list_users[ann1.pk] = ann1
                    if ann2.pk not in self._list_users.keys():
                        self._list_users[ann2.pk] = ann2
            if test:
                if combine:
                    self.texts = df
                else:
                    self.texts[txt.pk] = df
            if gold_standard:
                if combine:
                    self.texts_gold = self.precision_recall_calc(
                        txt, gold_standard, user_group, anno_proj=anno_proj)
                else:
                    self.texts_gold[txt.pk] = self.precision_recall_calc(
                        txt, gold_standard, user_group, anno_proj=anno_proj)



from rest_framework import serializers

from apis_core.apis_entities.serializers import (
    PersonSerializer,
    PlaceSerializer
)
from apis_core.apis_vocabularies.serializers import (
    PersonPlaceRelationSerializer,
)
from .models import (InstitutionInstitution, PersonInstitution, PersonPlace, PersonPerson,
                     PersonEvent, PersonWork, InstitutionPlace, InstitutionEvent, InstitutionWork,
                     PlaceEvent, PlaceWork, PlacePlace, EventWork, EventEvent, WorkWork)


class InstitutionInstitutionSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionInstitution


class PersonInstitutionSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonInstitution


class PersonPlaceSerializer(serializers.HyperlinkedModelSerializer):

    # url = serializers.HyperlinkedIdentityField(
    #     view_name="apis:apis_api:personplace-detail",
    #     lookup_field="pk"
    # )
    url = serializers.HyperlinkedIdentityField(
        view_name="apis:apis_api:personplace-detail",
        lookup_field="pk"
    )
    related_person = PersonSerializer()
    related_place = PlaceSerializer()
    relation_type = PersonPlaceRelationSerializer()
    # related_place__name = serializers.CharField()

    class Meta:
        fields = [
            'url',
            'id',
            'related_person',
            'related_place',
            'start_date',
            'start_date_written',
            'end_date',
            'end_date_written',
            'relation_type',
        ]
        model = PersonPlace


class PersonPersonSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonPerson


class PersonEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonEvent


class PersonWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonWork


class InstitutionPlaceSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="apis:apis_api:institutionplace-detail",
        lookup_field="pk"
    )

    class Meta:
        fields = '__all__'
        model = InstitutionPlace


class InstitutionEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionEvent


class InstitutionWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionWork


class PlaceEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlaceEvent


class PlaceWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlaceWork


class PlacePlaceSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlacePlace


class EventWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = EventWork


class EventEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = EventEvent


class WorkWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = WorkWork



from crispy_forms.bootstrap import Accordion, AccordionGroup
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit, Layout
from dal import autocomplete
from django import forms

from .models import *


class UriForm(forms.ModelForm):
    class Meta:
        model = Uri
        fields = "__all__"
        widgets = {
            'entity': autocomplete.ModelSelect2(
                url='apis_core:apis_metainfo-ac:apis_tempentity-autocomplete'),
            }

    def __init__(self, *args, **kwargs):
        super(UriForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-3'
        self.helper.field_class = 'col-md-9'
        self.helper.add_input(Submit('submit', 'save'),)


class UriFilterFormHelper(FormHelper):
    def __init__(self, *args, **kwargs):
        super(UriFilterFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = 'genericFilterForm'
        self.form_method = 'GET'
        self.helper.form_tag = False
        self.add_input(Submit('Filter', 'Search'))
        self.layout = Layout(
            Accordion(
                AccordionGroup(
                    'Filter',
                    'uri',
                    'domain',
                    'entity__name',
                    css_id="basic_search_fields"
                    ),
                )
            )



def calculate_age(row, age_field):
    try:
        year = row['year']
    except ValueError:
        year = None
    try:
        age = int(str(row[age_field])[:4])
    except ValueError:
        age = None
    if year and age:
        return year-age
    else:
        return 0



#!/usr/bin/python
# -*- coding: utf-8 -*-
import re
import types

import rdflib
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import FieldError
from rdflib import XSD

from apis_core.apis_labels.models import Label
from apis_core.apis_metainfo.models import Collection, Uri
from apis_core.apis_metainfo.models import Uri as genUri
from apis_core.apis_vocabularies.models import (
    LabelType)
from apis_core.default_settings.RDF_settings import sett_RDF_generic


def harmonize_geonames_id(uri):

    """checks if a geonames Url points to geonames' rdf expression"""

    if 'geonames' in uri:
        geo_id = "".join(re.findall(r'\d', uri))
        return "http://sws.geonames.org/{}/".format(geo_id)
    else:
        return uri


class GenericRDFParser(object):
    """A generic class for parsing RDFs to the APIS data model and save
    objects to the db.

    Attributes:

    - self.objct: (object) the object created by the parser

    - self.labels: (list) list of labels created by the parser (only saved when self.save() is called)

    - self.related_objcts: (list) list of related objects (relations) (only saved when self.save() is called)

    - self.kind: (string) kind of entity of the object (Persion, Place, Institution, Work, Event)

    - self.uri: (string) uri provided when initializing the object

    - self.saved: (boolean) indicates whether the object was saved to the db

    - self.created: (boolean) indicates whether the object was created
    """

    def save(self):
        """
        :return: django object saved to db or False if nothing was saved
        """
        if not self.created:
            return False

        exist = genUri.objects.filter(uri=self.uri)
        if exist.count() > 0:
            if exist[0].entity is not None:
                return exist[0].entity
        self.objct.status = 'distinct'
        self.objct.save()
        def_coll, created = Collection.objects.get_or_create(name='Default import collection')
        self.objct.collection.add(def_coll)
        self.saved = True
        self._objct_uri.entity=self.objct
        self._objct_uri.save()
        for lab in self.labels:
            lab.temp_entity = self.objct
            lab.save()
        for obj in self.related_objcts:
            if hasattr(obj, 'related_'+self.kind.lower()+'B'):
                setattr(obj, 'related_' + self.kind.lower() + 'A_id', self.objct.pk)
            else:
                setattr(obj, 'related_'+self.kind.lower()+'_id', self.objct.pk)
            obj.save()
        return self.objct

    def merge(self, m_obj, app_label_relations='apis_relations'):
        """
        :param m_obj: the object to merge with (must be an django model object instance)
        :param app_label_relations: (string) the label of the Django app that contains the relations
        :return: django object saved to db or False if nothing was saved
        """
        for rel in ContentType.objects.filter(app_label=app_label_relations, model__icontains=self.kind.lower()):
            rel_q = {'related_' + self.kind.lower(): m_obj}
            rel2 = rel.model_class()
            try:
                for rel_exst in rel2.objects.filter(**rel_q):
                    setattr(rel_exst, 'related_'+self.kind.lower()+'_id', self.objct.pk)
                    rel_exst.save()
            except FieldError:  # e.g. PlacePlace relations have different related_ fields
                rel_q = {'related_' + self.kind.lower()+'A': m_obj}
                for rel_exst in rel2.objects.filter(**rel_q):
                    setattr(rel_exst, 'related_'+self.kind.lower()+'A_id', self.objct.pk)
                    rel_exst.save()
                rel_q = {'related_' + self.kind.lower() + 'B': m_obj}
                for rel_exst in rel2.objects.filter(**rel_q):
                    setattr(rel_exst, 'related_'+self.kind.lower()+'B_id', self.objct.pk)
                    rel_exst.save()
        for z in genUri.objects.filter(entity=m_obj):
            z.entity_id = self.objct.pk
            z.save()
        for z in Label.objects.filter(temp_entity=m_obj):
            z.temp_entity_id = self.objct.pk
            z.save()
        if hasattr(m_obj, 'first_name'):
            legacy_name = '{}, {}'.format(m_obj.name, m_obj.first_name)
        else:
            legacy_name = m_obj.name
        lt, created = LabelType.objects.get_or_create(name='legacy name')
        Label.objects.create(temp_entity_id=self.objct.pk, label=legacy_name, label_type=lt)
        for col in m_obj.collection.all():
            self.objct.collection.add(col)
        if 'apis_highlighter' in settings.INSTALLED_APPS:
            for ann in m_obj.annotation_set.all():  # Todo: check if this works now with highlighter
                ann.entity_link = self.objct
                ann.save()
        for txt in m_obj.text.all():
            self.objct.text.add(txt)
        if m_obj.source:
            self.objct.source = m_obj.source
            self.objct.save()
        m_obj.delete()
        return self.objct

    def get_or_create(self):
        """
        :return: Returns the parsed object. Saves it to the db when needed
        """
        if not self.created:
            return self.objct
        else:
            if not self.saved:
                ob = self.save()
                return ob
            else:
                return self.objct

    def __init__(self, uri, kind, app_label_entities="apis_entities",
                 app_label_relations="apis_relations", app_label_vocabularies="apis_vocabularies", **kwargs):
        """
        :param uri: (url) Uri to parse the object from (http://test.at). The uri must start with a base url mentioned in the RDF parser settings file.
        :param kind: (string) Kind of entity (Person, Place, Institution, Work, Event)
        :param app_label_entities: (string) Name of the Django app that contains the entities that we create.
        :param app_label_relations: (string) Name of the Django app that contains the relations for the merging process.
        :param app_label_vocabularies: (string) Name of the Django app that contains the vocabularies defining the entities and relations.
        """
        owl = "http://www.w3.org/2002/07/owl#"

        def exist(uri, create_uri=False):
            print(uri)
            print(type(uri))
            if objct.objects.filter(uri__uri=uri).count() > 0:
                return True, objct.objects.get(uri__uri=uri)
            elif create_uri:
                self._objct_uri = genUri.objects.create(uri=uri)
                return False, False
            else:
                return False, False

        def prep_string(tupl):
            if isinstance(tupl, str):
                res = tupl
            elif tupl[1]:
                m = re.match(tupl[1][0], tupl[0])
                group = tupl[1][1]
                if not group:
                    group = 0
                try:
                    res = m.group(group)
                except:
                    res = tupl[0]
            else:
                res = tupl[0]
            if len(res) > 255:
                res = res[:250] + '...'
            return res.strip()
        objct = ContentType.objects.get(app_label=app_label_entities, model=kind.lower()).model_class()
        force = kwargs.get('force', None)
        res_attrb = dict()
        labels = []
        related_objcts = []
        uri = harmonize_geonames_id(uri)
        self.uri = uri
        self.kind = kind
        self.saved = False
        test = exist(self.uri, create_uri=True)
        if test[0] and not force:
            self.objct = test[1]
            self.created = False
        else:
            self.created = True
            rdf_t = dict()
            for x in sett_RDF_generic[kind]['data']:
                self.settings_defined = False
                if not uri.startswith(x['base_url']):
                    continue
                self.settings_defined = True
                g = rdflib.Graph()
                uri_2 = uri
                if not uri_2.endswith('/'):
                    uri_2 += '/'
                o2 = rdflib.term.Literal(uri, datatype=XSD.string)
                duri = rdflib.term.URIRef('http://d-nb.info/standards/elementset/dnb#deprecatedUri')
                #g.parse('{}{}'.format(uri_2.strip(), x['url_appendix']), format='xml')
                g.parse(uri)
                list_sameas = []
                if (None, duri, o2) in g:
                    o2 = g.value(subject=None, predicate=duri, object=o2, any=True)
                    ex2 = exist(o2, create_uri=False)
                    if ex2[0]:
                        self.objct = ex2[1]
                        self.created = False
                        if Uri.objects.filter(uri=uri_2).count() == 0:
                            Uri.objects.create(uri=uri_2, entity=self.objct)
                        break
                    else:
                        list_sameas.append(str(o2))
                else:
                    o2 = rdflib.term.URIRef(uri)
                sameas = rdflib.term.URIRef(owl+'sameAs')
                for p in g.objects(subject=o2, predicate=sameas):
                    list_sameas.append(genUri(uri=p))
                self.sameas = list_sameas
                if 'kind' in x.keys():
                    for k in x['kind']:
                        kind_rdf = rdflib.term.URIRef(k[0])
                        kind_val = g.value(o2, kind_rdf)
                        if kind_val is not None:
                            break
                        else:
                            kind_val = k[1]
                    if kind_val is not None:
                        kind_objct = ContentType.objects.get(
                        app_label=app_label_vocabularies, model=kind.lower() + 'Type'.lower()).model_class()
                        kind_objct, created = kind_objct.objects.get_or_create(name=kind_val)
                        res_attrb['kind'] = kind_objct
                for uri_2 in list_sameas:
                    test = exist(uri_2)
                    if test[0]:
                        self.objct = test[1]
                        self.created = False
                        uri_3 = genUri(uri=uri, entity=self.objct)
                        uri_3.save()
                for xx in x['attributes']:
                    rdf_t[xx['name']] = ()
                    subj2 = []
                    results = []
                    ind_type = ()
                    for z in xx['identifiers']:
                        if len(results) > 0:
                            continue
                        cnt = 0
                        cnt_2 = 1
                        try:
                            k = z[cnt_2]
                        except:
                            k = '='
                        subj = [o2, ]
                        while k:
                            for indx, s in enumerate(subj):
                                if z[cnt][0] == 'objects':
                                    pred = rdflib.term.URIRef(z[cnt][2])
                                    res = g.objects(subject=s, predicate=pred)
                                    if not isinstance(res, types.GeneratorType):
                                        break
                                    for r in res:
                                        if z[cnt][3]:
                                            if not getattr(r, z[cnt][3][0]) == z[cnt][3][1]:
                                                continue
                                        if k == '>':
                                            subj2.append(r)
                                        elif k == '=':
                                            results.append((z[cnt][1], r, indx))
                                            ind_type += ((len(ind_type), z[cnt][1]),)
                                            k = False
                            cnt_2 += 2
                            try:
                                k = z[cnt_2]
                            except:
                                k = '='
                            if cnt + 2 > len(z):
                                k = None
                            cnt += 2
                            subj = subj2
                    print(results)
                    for attrb in sett_RDF_generic[kind]['matching']['attributes'].keys():
                        res_2 = []
                        for x in sett_RDF_generic[kind]['matching']['attributes'][attrb]:
                            print('new print: {}'.format(x))
                            for s in x:
                                for ind, elem in filter(lambda x: x[1] == s[0], ind_type):
                                    elem = results[ind][1]
                                    res_2.append(prep_string((elem, s[1])))
                                if isinstance(s, str):
                                    res_2.append(s)
                            if len(res_2) == len(x):
                                res_attrb[attrb] = ''.join(res_2)
                    print(res_attrb)
                    for lab in sett_RDF_generic[kind]['matching']['labels'].keys():
                        lb_type, created = LabelType.objects.get_or_create(name=lab)
                        for x in sett_RDF_generic[kind]['matching']['labels'][lab]:
                            for ind, elem in filter(lambda a: a[1]==x[0], ind_type):
                                elem = results[ind][1]
                                lb = Label(label=prep_string((elem, x[1])), isoCode_639_3=elem.language, label_type=lb_type)
                                labels.append(lb)
                    if kwargs.get('drill_down', True):
                        for con in sett_RDF_generic[kind]['matching']['linked objects']:
                            for x in con['object']:
                                for ind, elem in filter(lambda a: a[1]==x[0], ind_type):
                                    elem = results[ind][1]
                                    ob = GenericRDFParser(elem, con['type'], drill_down=False)
                                    if ob.created and not ob.saved:
                                        ob.save()   # TODO: We should move the save of related objects in the save routine
                                    try:
                                        u = ContentType.objects.get(app_label=app_label_relations, model=kind.lower()+con['type'].lower())
                                        u_kind = ContentType.objects.get(app_label=app_label_vocabularies, model=kind.lower()+con['type'].lower()+'Relation'.lower())
                                    except ContentType.DoesNotExist:
                                        u = ContentType.objects.get(app_label=app_label_relations, model=con['type'].lower()+kind.lower())
                                        u_kind = ContentType.objects.get(app_label=app_label_vocabularies, model=con['type'].lower()+kind.lower()+'Relation'.lower())
                                    u_kind_2 = u_kind.model_class()
                                    u2 = u.model_class()()
                                    uk, created = u_kind_2.objects.get_or_create(name=con['kind'])
                                    if con['type'] == kind:
                                        setattr(u2, 'related_' + con['type'].lower() + 'B_id', ob.objct.pk)
                                    else:
                                        setattr(u2, 'related_' + con['type'].lower() + '_id', ob.objct.pk)
                                    setattr(u2, 'relation_type_id', uk.pk)
                                    related_objcts.append(u2)
            if self.created:
                self.objct = objct(**res_attrb)
                self.labels = labels
                self.related_objcts = related_objcts



import copy
import re

import yaml
from crispy_forms.bootstrap import Accordion, AccordionGroup
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout
from dal import autocomplete
from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db.models import Q
from django.urls import reverse
# import autocomplete_light.shortcuts as al
from django.utils.translation import ugettext_lazy as _

from apis_core.apis_entities.fields import ListSelect2
from apis_core.apis_entities.models import AbstractEntity
# from dal.autocomplete import ListSelect2
from apis_core.apis_metainfo.models import TempEntityClass, Text, Uri
from apis_core.apis_relations.models import AbstractRelation
from apis_core.helper_functions import DateParser
from apis_core.helper_functions.RDFParser import RDFParser, APIS_RDF_URI_SETTINGS
from .tables import get_generic_relations_table

# from dal.autocomplete import ListSelect2

if 'apis_highlighter' in settings.INSTALLED_APPS:
    from apis_highlighter.models import Annotation, AnnotationProject


def validate_target_autocomplete(value):
    try:
        value = int(value)
    except ValueError:
        if value.startswith('http'):
            test = False
            sett = yaml.load(open(APIS_RDF_URI_SETTINGS, 'r'))
            regx = [x['regex'] for x in sett['mappings']]
            regx.append('http.*oeaw\.ac\.at')
            for k, v in getattr(settings, 'APIS_AC_INSTANCES', {}).items():
                regx.append(v['url'].replace('.', '\.'))
            for r in regx:
                if re.match(r, value):
                    test = True
            if not test:
                if Uri.objects.filter(uri=value).count() != 1:
                    raise ValidationError(
                        _('Invalid value: %(value)s, the url you are using is not configured'),
                        code='invalid',
                        params={'value': value},
                    )
        else:
            raise ValidationError(
                _('Invalid value: %(value)s, use either URLs or select a value'),
                code='invalid',
                params={'value': value},
            )


class GenericRelationForm(forms.ModelForm):

    class Meta:
        model = TempEntityClass
        fields = ['start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
        }

    def save(self, site_instance, instance=None, commit=True):
        """
        Save function of the GenericRelationForm.
        :param site_instance: Instance where the form is used on
        :param instance: PK of the relation that is saved
        :param commit: Whether to already commit the save.
        :type site_instance: object
        :type instance: int
        :type commit: bool
        :rtype: object
        :return: instance of relation
        """
        cd = self.cleaned_data
        if instance:
            x = self.relation_form.objects.get(pk=instance)
        else:
            x = self.relation_form()
        x.relation_type_id = cd['relation_type']
        x.start_date_written = cd['start_date_written']
        x.end_date_written = cd['end_date_written']
        x.notes = cd['notes']
        x.references = cd['references']
        setattr(x, self.rel_accessor[3], site_instance)
        target = AbstractEntity.get_entity_class_of_name(self.rel_accessor[0])
        t1 = target.get_or_create_uri(cd['target'])
        if not t1:
            t1 = RDFParser(cd['target'], self.rel_accessor[0]).get_or_create()
        setattr(x, self.rel_accessor[2], t1)
        if self.highlighter:
            an_proj = AnnotationProject.objects.get(pk=int(self.request.session.get('annotation_project', 1)))
            x.published = an_proj.published
        if commit:
            x.save()
        if self.highlighter:
            if not commit:
                x.save()
            txt = Text.objects.get(pk=cd['HL_text_id'][5:])
            a = Annotation(
                start=cd['HL_start'],
                end=cd['HL_end'],
                text=txt,
                user_added=self.request.user,
                annotation_project_id=int(self.request.session.get('annotation_project', 1)))
            a.entity_link = x
            a.save()
        print('saved: {}'.format(x))
        return x

    def get_text_id(self):
        """
        Function to retrieve the highlighted text.
        :return: ID of text that was highlighted
        """
        return self.cleaned_data['HL_text_id'][5:]

    def get_html_table(self, entity_type, request, site_instance, form_match):

        table = get_generic_relations_table(relation_class=self.relation_form, entity_instance=site_instance, detail=False)
        prefix = re.match(r'([A-Z][a-z])[^A-Z]*([A-Z][a-z])', self.relation_form.__name__)
        prefix = prefix.group(1)+prefix.group(2)+'-'
        if form_match.group(1) == form_match.group(2):
            dic_a = {'related_'+entity_type.lower()+'A': site_instance}
            dic_b = {'related_' + entity_type.lower() + 'B': site_instance}
            if 'apis_highlighter' in settings.INSTALLED_APPS:
                objects = self.relation_form.objects.filter_ann_proj(request=request).filter(
                    Q(**dic_a) | Q(**dic_b)
                )
            else:
                objects = self.relation_form.objects.filter(
                    Q(**dic_a) | Q(**dic_b)
                )

            table_html = table(data=objects, prefix=prefix)
        else:
            tab_query = {'related_'+entity_type.lower(): site_instance}
            if 'apis_highlighter' in settings.INSTALLED_APPS:
                ttab = self.relation_form.objects.filter_ann_proj(
                    request=request).filter(**tab_query)
            else:
                ttab = self.relation_form.objects.filter(**tab_query)
            table_html = table(data=ttab, prefix=prefix)
        return table_html

    def __init__(self, siteID=None, highlighter=False, *args, **kwargs):
        """
        Generic Form for relations.
        :param siteID: ID of the entity the form is used on
        :param entity_type: Entity type of the entity the form is used on
        :param relation_form: Type of relation form.
        :param instance: instance of relation.
        :param highlighter: whether the form is used in the highlighter
        :type siteID: int
        :type entity_type: object or int
        :type relation_form: object or int
        :type instance: object
        :type highlighter: bool
        """
        attrs = {'data-placeholder': 'Type to get suggestions',
                 'data-minimum-input-length': getattr(settings, "APIS_MIN_CHAR", 3),
                 'data-html': True,
                 'style': 'width: 100%'}
        help_text_target = "Search and select or use an URL from a reference resource"
        attrs_target = copy.deepcopy(attrs)
        attrs_target['data-tags'] = '1'
        css_notes = 'LS'
        self.highlighter = highlighter
        entity_type = kwargs.pop('entity_type')
        if type(entity_type) != str:
            entity_type = entity_type.__name__
        self.relation_form = kwargs.pop('relation_form')
        if type(self.relation_form) == str:
            self.relation_form = AbstractRelation.get_relation_class_of_name(self.relation_form)
        self.request = kwargs.pop('request', False)
        super(GenericRelationForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'] = forms.CharField(label='Relation type', required=True)
        self.helper = FormHelper()
        self.helper.form_class = '{}Form'.format(str(self.relation_form))
        self.helper.form_tag = False
        lst_src_target = re.findall('[A-Z][^A-Z]*', self.relation_form.__name__)
        if lst_src_target[0] == lst_src_target[1]:
            if instance and instance.id:
                if getattr(instance, 'related_{}A_id'.format(lst_src_target[0].lower())) == int(siteID):
                    self.rel_accessor = (lst_src_target[1], True,
                                         'related_{}B'.format(lst_src_target[1].lower()),
                                         'related_{}A'.format(lst_src_target[0].lower()))
                else:
                    self.rel_accessor = (lst_src_target[1], False,
                                         'related_{}A'.format(lst_src_target[1].lower()),
                                         'related_{}B'.format(lst_src_target[0].lower()))
            else:
                self.rel_accessor = (lst_src_target[1], True,
                                     'related_{}B'.format(lst_src_target[1].lower()),
                                     'related_{}A'.format(lst_src_target[0].lower()))
            self.fields['relation_type'] = autocomplete.Select2ListCreateChoiceField(
                label='Relation type',
                widget=ListSelect2(
                    #url='/vocabularies/autocomplete/{}{}relation/normal'.format(lst_src_target[0].lower(), lst_src_target[1].lower()),
                    url=reverse('apis:apis_vocabularies:generic_vocabularies_autocomplete', args=[''.join([lst_src_target[0].lower(), lst_src_target[1].lower(), 'relation']), 'normal']),
                    attrs=attrs))
            self.fields['target'] = autocomplete.Select2ListCreateChoiceField(
                label=lst_src_target[1],
                widget=ListSelect2(
                    #url='/entities/autocomplete/{}'.format(lst_src_target[1].lower()),
                    url = reverse('apis:apis_entities:generic_entities_autocomplete', args=[lst_src_target[1].lower()]),
                    attrs=attrs_target),
                validators=[validate_target_autocomplete],
                help_text=help_text_target)
        elif entity_type.lower() == lst_src_target[0].lower():
            self.rel_accessor = (lst_src_target[1], True,
                                 'related_{}'.format(lst_src_target[1].lower()),
                                 'related_{}'.format(lst_src_target[0].lower()))
            self.fields['relation_type'] = autocomplete.Select2ListCreateChoiceField(
                label='Relation type',
                widget=ListSelect2(
                    #url='/vocabularies/autocomplete/{}{}relation/normal'.format(lst_src_target[0].lower(), lst_src_target[1].lower()),
                    url=reverse('apis:apis_vocabularies:generic_vocabularies_autocomplete', args=[''.join([lst_src_target[0].lower(), lst_src_target[1].lower(), 'relation']), 'normal']),
                    attrs=attrs))
            self.fields['target'] = autocomplete.Select2ListCreateChoiceField(
                label=lst_src_target[1],
                widget=ListSelect2(
                    #url='/entities/autocomplete/{}'.format(lst_src_target[1].lower()),
                    url = reverse('apis:apis_entities:generic_entities_autocomplete', args=[lst_src_target[1].lower()]),
                    attrs=attrs_target),
                validators=[validate_target_autocomplete],
                help_text=help_text_target)
        elif entity_type.lower() == lst_src_target[1].lower():
            self.rel_accessor = (lst_src_target[0], False,
                                 'related_{}'.format(lst_src_target[0].lower()),
                                 'related_{}'.format(lst_src_target[1].lower()))
            self.fields['relation_type'] = autocomplete.Select2ListCreateChoiceField(
                label='Relation type',
                widget=ListSelect2(
                    #url='/vocabularies/autocomplete/{}{}relation/reverse'.format(lst_src_target[0].lower(), lst_src_target[1].lower()),
                    url=reverse('apis:apis_vocabularies:generic_vocabularies_autocomplete', args=[''.join([lst_src_target[0].lower(), lst_src_target[1].lower(), 'relation']), 'reverse']),
                    attrs=attrs))
            self.fields['target'] = autocomplete.Select2ListCreateChoiceField(
                label=lst_src_target[0],
                widget=ListSelect2(
                    #url='/entities/autocomplete/{}'.format(lst_src_target[0].lower()),
                    url = reverse('apis:apis_entities:generic_entities_autocomplete', args=[lst_src_target[0].lower()]),
                    attrs=attrs_target),
                validators=[validate_target_autocomplete],
                help_text=help_text_target)
        else:
            print('no hit rel_accessor')
        if instance and instance.id:
            self.fields['target'].choices = [
                (str(Uri.objects.filter(entity=getattr(instance, self.rel_accessor[2]))[0]),
                 str(getattr(instance, self.rel_accessor[2])))]
            self.fields['target'].initial = (str(Uri.objects.filter(entity=getattr(instance, self.rel_accessor[2]))[0]),
                                             str(getattr(instance, self.rel_accessor[2])))
            if self.rel_accessor[1]:
                self.fields['relation_type'].choices = [(instance.relation_type.id,
                                                         instance.relation_type.label)]
                self.fields['relation_type'].initial = (instance.relation_type.id, instance.relation_type.label)
            else:
                self.fields['relation_type'].choices = [(instance.relation_type.id,
                                                         instance.relation_type.label_reverse)]
                self.fields['relation_type'].initial = (instance.relation_type.id, instance.relation_type.label_reverse)
        if highlighter:
            css_notes = 'HL'

        self.helper.include_media = False
        self.helper.layout = Layout(
            'relation_type',
            'target',
            'start_date_written',
            'end_date_written',
            Accordion(
                AccordionGroup(
                    'Notes and References',
                    'notes',
                    'references',
                    active=False,
                    css_id="{}_{}_notes_refs".format(self.relation_form.__name__, css_notes))))

        if highlighter:
            self.fields['HL_start'] = forms.IntegerField(widget=forms.HiddenInput)
            self.fields['HL_end'] = forms.IntegerField(widget=forms.HiddenInput)
            self.fields['HL_text_id'] = forms.CharField(widget=forms.HiddenInput)
            self.helper.layout.extend([
                'HL_start',
                'HL_end',
                'HL_text_id'])


        if instance != None:

            if instance.start_date_written:
                self.fields['start_date_written'].help_text = DateParser.get_date_help_text_from_dates(
                    single_date=instance.start_date,
                    single_start_date=instance.start_start_date,
                    single_end_date=instance.start_end_date,
                    single_date_written=instance.start_date_written,
                )
            else:
                self.fields['start_date_written'].help_text = DateParser.get_date_help_text_default()

            if instance.end_date_written:
                self.fields['end_date_written'].help_text = DateParser.get_date_help_text_from_dates(
                    single_date=instance.end_date,
                    single_start_date=instance.end_start_date,
                    single_end_date=instance.end_end_date,
                    single_date_written=instance.end_date_written,
                )
            else:
                self.fields['end_date_written'].help_text = DateParser.get_date_help_text_default()



import re

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db.models.query import QuerySet
from django.urls import reverse
from rest_framework import serializers
from reversion.models import Version

from apis_core.apis_labels.serializers import LabelSerializerLegacy as LabelSerializer

base_uri = getattr(settings, "APIS_BASE_URI", "http://apis.info")
if base_uri.endswith("/"):
    base_uri = base_uri[:-1]


class CollectionSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    name = serializers.CharField()


class VocabsSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    name = serializers.CharField()
    label = serializers.CharField()


class EntityUriSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    uri = serializers.URLField()


class TextSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    kind = VocabsSerializer()
    text = serializers.CharField()


class EntitySerializer(serializers.Serializer):
    id = serializers.IntegerField()
    url = serializers.SerializerMethodField(method_name="add_url")
    name = serializers.CharField()
    start_date = serializers.DateField()
    end_date = serializers.DateField()
    uris = EntityUriSerializer(source="uri_set", many=True)
    labels = LabelSerializer(source="label_set", many=True)
    revisions = serializers.SerializerMethodField(method_name="add_revisions")

    def add_revisions(self, obj):
        ver = Version.objects.get_for_object(obj)
        res = []
        for v in ver:
            usr_1 = getattr(v.revision, "user", None)
            if usr_1 is not None:
                usr_1 = usr_1.username
            else:
                usr_1 = "Not specified"
            res.append(
                {
                    "id": v.id,
                    "date_created": v.revision.date_created,
                    "user_created": usr_1,
                }
            )
        return res

    def add_relations(self, obj):
        res = {}
        mk = obj.__class__.__name__
        for rel in ContentType.objects.filter(
            app_label="apis_relations", model__icontains=mk.lower()
        ):
            mk2 = re.match(r"{}([A-Za-z]+)".format(mk.lower()), rel.model)
            reverse = False
            if not mk2:
                mk2 = re.match(r"([A-Za-z]+){}".format(mk.lower()), rel.model)
                reverse = True
            res["{}s".format(mk2.group(1))] = []
            if mk2.group(1).lower() != mk.lower():
                if self._only_published:
                    rel_qs = (
                        getattr(obj, "{}_set".format(rel.model)).all().filter_for_user()
                    )
                else:
                    rel_qs = getattr(obj, "{}_set".format(rel.model)).all()
                for rel2 in rel_qs:
                    res["{}s".format(mk2.group(1))].append(
                        RelationEntitySerializer(
                            rel2,
                            own_class=mk,
                            read_only=True,
                            context=self.context,
                            reverse=reverse,
                        ).data
                    )
            else:
                for t in ["A", "B"]:
                    for rel2 in (
                        getattr(obj, "related_{}{}".format(mk.lower(), t))
                        .all()
                        .filter_for_user()
                    ):
                        if t == "A":
                            ok = "{}B".format(mk.lower())
                            reverse = True
                        else:
                            ok = "{}A".format(mk.lower())
                            reverse = False
                        res["{}s".format(mk2.group(1))].append(
                            RelationEntitySerializer(
                                rel2,
                                own_class=ok,
                                read_only=True,
                                context=self.context,
                                reverse=reverse,
                            ).data
                        )
        return res

    def add_entity_type(self, obj):
        return str(obj.__class__.__name__)

    def add_url(self, obj):
        url = f"{base_uri}{reverse('GetEntityGenericRoot', kwargs={'pk': obj.pk})}"
        return url

    def __init__(
        self, *args, depth_ent=1, only_published=True, add_texts=False, **kwargs
    ):
        super(EntitySerializer, self).__init__(*args, **kwargs)
        self._only_published = only_published
        if type(self.instance) == QuerySet:
            inst = self.instance[0]
        else:
            inst = self.instance
        if inst is None:
            return
        for f in inst._meta.fields:
            field_name = re.search(r"([A-Za-z]+)\'>", str(f.__class__)).group(1)
            if field_name in [
                "CharField",
                "DateField",
                "DateTimeField",
                "IntegerField",
                "FloatField",
            ]:
                self.fields[f.name] = getattr(serializers, field_name)()
            elif field_name in ["ForeignKey", "ManyToMany"]:
                if str(f.related_model.__module__).endswith("apis_vocabularies.models"):
                    many = False
                    if f.many_to_many or f.one_to_many:
                        many = True
                    self.fields[f.name] = VocabsSerializer(many=many)
        for f in inst._meta.many_to_many:
            if f.name.endswith("relationtype_set"):
                continue
            elif f.name == "collection":
                self.fields["collection"] = CollectionSerializer(many=True)
            elif str(f.related_model.__module__).endswith("apis_vocabularies.models"):
                self.fields[f.name] = VocabsSerializer(many=True)
        self.fields["entity_type"] = serializers.SerializerMethodField(
            method_name="add_entity_type"
        )
        if depth_ent == 1:
            self.fields["relations"] = serializers.SerializerMethodField(
                method_name="add_relations"
            )
        if add_texts:
            self.fields["text"] = TextSerializer(many=True)


class RelationEntitySerializer(serializers.Serializer):
    id = serializers.IntegerField()
    start_date = serializers.DateField()
    end_date = serializers.DateField()
    start_date_written = serializers.DateField()
    end_date_written = serializers.DateField()
    relation_type = serializers.SerializerMethodField(method_name="add_relation_label")
    annotation = serializers.SerializerMethodField(method_name="add_annotations")
    revisions = serializers.SerializerMethodField(method_name="add_revisions")

    def add_revisions(self, obj):
        ver = Version.objects.get_for_object(obj)
        res = []
        for v in ver:
            usr_1 = getattr(v.revision, "user", None)
            if usr_1 is not None:
                usr_1 = usr_1.username
            else:
                usr_1 = "Not specified"
            res.append(
                {
                    "id": v.id,
                    "date_created": v.revision.date_created,
                    "user_created": usr_1,
                }
            )
        return res

    def add_annotations(self, obj):
        if "apis_highlighter" in settings.INSTALLED_APPS:
            res = []
            offs = 50
            for an in obj.annotation_set.all():
                r1 = dict()
                r1["id"] = an.pk
                r1["user"] = an.user_added.username
                text = an.text.text
                if offs < an.start:
                    s = an.start - offs
                else:
                    s = 0
                if offs + an.end < len(text):
                    e = an.end + offs
                else:
                    e = len(text)
                r1["annotation"] = text[an.start : an.end]
                r1["text"] = text[s:e]
                r1["text"] = "{}<annotation>{}</annotation>{}".format(
                    r1["text"][: an.start - s],
                    r1["text"][an.start - s : an.end - s],
                    r1["text"][an.end - s :],
                )
                r1["text"] = r1["text"].replace("\r\n", "<br/>")
                r1["text"] = r1["text"].replace("\r", "<br/>")
                r1["text"] = r1["text"].replace("\n", "<br/>")

                r1["string_offset"] = "{}-{}".format(an.start, an.end)
                # r1["text_url"] = self.context["request"].build_absolute_uri(
                #        reverse("apis_core:apis_api:text-detail", kwargs={"pk": an.text_id})
                # )
                r1[
                    "text_url"
                ] = f"{base_uri}{reverse('apis_core:apis_api:text-detail', kwargs={'pk': an.text_id})}"
                res.append(r1)
            return res

    def add_entity(self, obj):
        return EntitySerializer(
            getattr(obj, "related_{}".format(self.entity_type)), depth_ent=0
        ).data

    def add_relation_label(self, obj):
        cm = obj.__class__.__name__
        res_1 = dict()
        res_1["id"] = obj.relation_type.pk
        res_1[
            "url"
        ] = f"{base_uri}{reverse('apis_core:apis_api:{}relation-detail'.format(cm).lower(), kwargs={'pk': obj.relation_type.pk},)}"
        if self.reverse and len(obj.relation_type.label_reverse) > 0:
            res_1["label"] = obj.relation_type.label_reverse
        elif self.reverse:
            res_1["label"] = "({})".format(obj.relation_type.label)
        else:
            res_1["label"] = obj.relation_type.label
        return res_1

    def __init__(self, *args, own_class=None, reverse=False, **kwargs):
        super(RelationEntitySerializer, self).__init__(*args, **kwargs)
        self.own_class = own_class
        self.reverse = reverse
        if self.instance is not None:
            for f in self.instance._meta.fields:
                if f.name.startswith("related_"):
                    mk2 = f.name.replace("related_", "")

                    if mk2.lower() != own_class.lower():
                        self.entity_type = mk2
                        self.fields["target"] = serializers.SerializerMethodField(
                            method_name="add_entity"
                        )



from django.conf.urls import url
from django.urls import path

from apis_core.apis_entities.autocomplete3 import GenericVocabulariesAutocomplete
from apis_core.apis_vocabularies import views

app_name = 'apis_vocabularies'

urlpatterns = [
    path('download/<model_name>/', views.dl_vocabs_as_csv, name='dl-vocabs'),
    url(r'^autocomplete/(?P<vocab>[a-zA-Z0-9-]+)/(?P<direct>[a-zA-Z0-9-]+)/$',
        GenericVocabulariesAutocomplete.as_view(),
        name='generic_vocabularies_autocomplete'),
]



import django_filters
from dal import autocomplete
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.urls import reverse
import operator
from functools import reduce

from .models import AbstractRelation


# TODO __sresch__ : Change this whole module according to the same logic as in apis_core/apis_entities/filters.py

def get_excluded_fields(model):
    modelname = model.__name__
    base_list = getattr(settings, "APIS_RELATIONS_FILTER_EXCLUDE", [])
    rel_model_conf = getattr(settings, "APIS_RELATIONS", {})
    if "exclude" in rel_model_conf.keys():
        if isinstance(rel_model_conf["exclude"], list):
            base_list.extend(rel_model_conf["exclude"])
    if modelname in rel_model_conf.keys():
        if "exclude" in rel_model_conf[modelname].keys():
            if isinstance(rel_model_conf[modelname]["exclude"], list):
                base_list.extend(rel_model_conf[modelname]["exclude"])
    return set(base_list)


def get_included_fields(model):
    modelname = model.__name__
    rel_model_conf = getattr(settings, "APIS_RELATIONS", {})
    if modelname in rel_model_conf.keys():
        return rel_model_conf[modelname].get("include", False)
    else:
        return False



FIELD_TO_FILTER = {
    "ForeignKey": "MultipleChoiceFilter",
    "ManyToManyField": "MultipleChoiceFilter",
    "TextField": "CharFilter",
    "CharField": "CharFilter",
    "DateField": "DateFromToRangeFilter",
    "BooleanField": "BooleanFilter"
}


def get_field_dicts(model, include_parents=False):
    fields = [
        {
            "f_name": x.name,
            "f_v_name": getattr(x, 'verbose_name', None),
            "f_help_text": getattr(x, 'helptext', None),
            "f_class_name": "{}".format(x.__class__.__name__),
            "f_model": getattr(x, 'related_model', None)
        } for x in model._meta.get_fields(include_parents=include_parents)
    ]
    return fields


def get_filters(model, exclude=False, include=False, include_parents=False):
    filters = []
    field_dicts = get_field_dicts(model, include_parents=include_parents)
    for x in field_dicts:
        filters.append(x['f_name'])
        if x['f_model']:
            rel_fields = get_field_dicts(x['f_model'], include_parents)
            for y in rel_fields:
                if 'apis_relations' in "{}".format(y['f_model']):
                    pass
                else:
                    rel_field_name = "{}__{}".format(x['f_name'], y['f_name'])
                    filters.append(rel_field_name)
    if include:
        filters = [x for x in filters if x in include]
    elif exclude:
        for x in exclude:
            if x.startswith("*") and not x.endswith("*"):
                filters = [f for f in filters if not f.lower().endswith(x[1:].lower())]
            elif x.startswith("*") and x.endswith("*"):
                filters = [f for f in filters if not x[1:-1].lower() in f]
            elif not x.startswith("*") and x.endswith("*"):
                filters = [f for f in filters if not f.lower().startswith(x[:-1].lower())]
            else:
                filters = [f for f in filters if not x.lower() == f.lower()]
    return filters


def get_generic_relation_filter(entity):
    class GenericListFilter(django_filters.FilterSet):
        #search = django_filters.CharFilter(method='search_filter_method')

        def name_label_filter(self, queryset, name, value):
            """
            Filter for including the alternative names in the names search.\
            The types of labels included in the query are
            currently hardcoded in a list.

            :param queryset: queryset that the filters are applied on
            :param name: name of the attribute to filter on (not used as label types are hardcoded)
            :param value: value for the filter
            :return: filtered queryset
            """
            alternate_names = getattr(settings, "APIS_ALTERNATE_NAMES", ['alternative name'])
            res = []
            orig_value = value
            for n in ['name', 'label__label']:
                value = orig_value
                f = '{}__'.format(n)
                if value.startswith('"') and value.endswith('"'):
                    value = value[1:-1]
                else:
                    f += 'i'
                if value.startswith('*') and value.endswith('*'):
                    f += 'contains'
                    value = value[1:-1]
                elif value.startswith('*'):
                    f += 'endswith'
                    value = value[1:]
                elif value.endswith('*'):
                    f += 'startswith'
                    value = value[:-1]
                else:
                    f += 'exact'
                if n == 'label__label':
                    res.append(Q(**{f: value, 'label__label_type__name__in': alternate_names}))
                else:
                    res.append(Q(**{f: value}))
            return queryset.filter(res[0] | res[1]).distinct()

        def wildcard_filter(self, queryset, name, value):
            f = '{}__'.format(name)
            if value.startswith('"') and value.endswith('"'):
                value = value[1:-1]
            else:
                f += 'i'
            if value.startswith('*') and value.endswith('*'):
                f += 'contains'
                value = value[1:-1]
            elif value.startswith('*'):
                f += 'endswith'
                value = value[1:]
            elif value.endswith('*'):
                f += 'startswith'
                value = value[:-1]
            else:
                f += 'exact'
            return queryset.filter(**{f: value})

        def search_filter_method(self, queryset, name, value):
            cls = queryset.model.__name__
            sett_filters = getattr(settings, "APIS_RELATIONS", {})
            if cls.lower() in sett_filters.keys():
                filter_attr = sett_filters[cls.lower()].get("search", ["name"])
                query = reduce(operator.or_, [ Q(**{attr: value}) for attr in filter_attr ] )


        class Meta:
            model = AbstractRelation.get_relation_class_of_name(entity)
            fields = get_filters(
                model,
                exclude=get_excluded_fields(model),
                include=get_included_fields(model),
                include_parents=True
            )

        def __init__(self, *args, **kwargs):
            attrs = {'data-placeholder': 'Type to get suggestions',
                     'data-minimum-input-length': getattr(settings, "APIS_MIN_CHAR", 3),
                     'data-html': True}
            super(GenericListFilter, self).__init__(*args, **kwargs)
            for x in self.filters.keys():
                if type(self.filters[x].field).__name__ == "ModelChoiceField":
                    current_model_name = str(self.filters[x].queryset.model.__name__).lower()
                    current_qs = self.filters[x].queryset
                    if ContentType.objects.filter(app_label='apis_entities', model=current_model_name).count() > 0:
                        self.filters[x] = django_filters.ModelMultipleChoiceFilter(
                            field_name=x,
                            queryset=current_qs,
                            widget=autocomplete.ModelSelect2Multiple(
                                url=reverse(
                                    'apis:apis_entities:generic_network_entities_autocomplete',
                                    kwargs={
                                        'entity': current_model_name
                                    }
                                ),
                            )
                        )
                    elif ContentType.objects.filter(app_label='apis_vocabularies', model=current_model_name).count() > 0:
                        self.filters[x] = django_filters.ModelMultipleChoiceFilter(
                            field_name=x,
                            queryset=current_qs,
                            widget=autocomplete.ModelSelect2Multiple(
                                url=reverse(
                                    'apis:apis_vocabularies:generic_vocabularies_autocomplete',
                                    kwargs={
                                        'vocab': current_model_name,
                                        'direct': 'normal'
                                    }
                                ),
                            )
                        )
                if type(self.filters[x].field).__name__ == "DateField":
                    self.filters[x] = django_filters.DateFromToRangeFilter(
                        field_name=x,
                    )
                if type(self.filters[x].field).__name__ == "CharField":
                    self.filters[x] = django_filters.CharFilter(
                        lookup_expr='icontains',
                        field_name=x,
                    )
                if type(self.filters[x].field).__name__ == "ModelMultipleChoiceField":
                    current_model_name = str(self.filters[x].queryset.model.__name__).lower()
                    current_qs = self.filters[x].queryset
                    self.filters[x] = django_filters.ModelMultipleChoiceFilter(
                        field_name=x,
                        queryset=current_qs,
                        widget=autocomplete.ModelSelect2Multiple(
                            url=reverse(
                                'apis:apis_entities:generic_network_entities_autocomplete',
                                kwargs={
                                    'entity': current_model_name
                                }
                            ),
                        )
                    )
    return GenericListFilter



from apis_core.vocabularies.models import ProfessionType, Title
from autocomplete_light import shortcuts as al

from .models import Person, Place, Institution, Event, Work


class TitleAutocomplete(al.AutocompleteModelBase):
	search_fields=['name', 'abbreviation']
	model = Title
	attrs = {
        'data-autocomplete-minimum-characters': 2,
        'placeholder': 'Start typing to get suggestions',
    }

al.register(Title, TitleAutocomplete)


class ProfessionTypeAutocomplete(al.AutocompleteModelBase):
	search_fields=['name']
	model = ProfessionType
	attrs = {
        'data-autocomplete-minimum-characters': 2,
        'placeholder': 'Start typing to get suggestions',
    }

al.register(ProfessionType, ProfessionTypeAutocomplete)


class DB_PersonAutocomplete(al.AutocompleteModelBase):
    search_fields = ['name', 'first_name']
    model = Person
    attrs = {
        'data-autocomplete-minimum-characters': 2,
        'placeholder': 'Start typing to get suggestions',
    }

al.register(DB_PersonAutocomplete)


class DB_PlaceAutocomplete(al.AutocompleteModelBase):
    search_fields = ['name', ]
    model = Place
    attrs = {
        'data-autocomplete-minimum-characters': 2,
        'placeholder': 'Start typing to get suggestions',
    }

al.register(DB_PlaceAutocomplete)


class DB_InstitutionAutocomplete(al.AutocompleteModelBase):
    search_fields = ['name', ]
    model = Institution
    attrs = {
        'data-autocomplete-minimum-characters': 2,
        'placeholder': 'Start typing to get suggestions',
    }

al.register(DB_InstitutionAutocomplete)


class DB_EventAutocomplete(al.AutocompleteModelBase):
    search_fields = ['name', ]
    model = Event
    attrs = {
        'data-autocomplete-minimum-characters': 2,
        'placeholder': 'Start typing to get suggestions',
    }

al.register(DB_EventAutocomplete)


class DB_WorkAutocomplete(al.AutocompleteModelBase):
    search_fields = ['name', ]
    model = Work
    attrs = {
        'data-autocomplete-minimum-characters': 2,
        'placeholder': 'Start typing to get suggestions',
    }

al.register(DB_WorkAutocomplete)



import lxml.etree as ET
from django.core.management.base import BaseCommand
from tqdm import tqdm

from apis_core.apis_entities.models import Work
from apis_core.apis_tei.tei_utils import get_node_from_template, tei_header

class Command(BaseCommand):
    help = 'Command to serialize APIS Works to XML/TEI places.xml'

    def add_arguments(self, parser):
        parser.add_argument(
            '-l',
            '--limit',
            action='store_true',
            help='number of entities should be limited',
        )
        parser.add_argument(
            '-f',
            '--full',
            action='store_true',
            help='should related entities e.g. birth-places be fully serialized',
        )
        parser.add_argument(
            '--collection',
            help='which collection?',
        )
    
    def handle(self, *args, **kwargs):

        tei_doc = tei_header(title="ListBibl", ent_type="<listBibl/>")
        entity_list = tei_doc.xpath("//*[local-name() = 'listBibl']")[0] 

        if kwargs['full']:
            print("full is set")
            full = True
        else:
            print("simple")
            full = False
        
        if kwargs['collection']:
            try:
                col_id = int(kwargs['collection'])
            except ValueError:
                print(f"collection needs to be an integer and not: {kwargs['collection']}")
                return False
        
            items = Work.objects.filter(collection=col_id)
        else:
            items = Work.objects.all()
        if kwargs['limit']:
            items = items[:25]
        print(f"serialize {items.count()} Works")
        for res in tqdm(items, total=len(items)):
            item_node = get_node_from_template(
                'apis_tei/work.xml', res, full=full
            )
            entity_list.append(item_node)
        
        with open('listwork.xml', 'w') as f:
            print(ET.tostring(tei_doc).decode('utf-8'), file=f)
        print("done")



from django.shortcuts import render, redirect, get_object_or_404
from django.urls import reverse_lazy
from django.views import generic
from django.views.generic.edit import DeleteView

from .forms import LabelForm
from .models import Label


class LabelListView(generic.ListView):
	model = Label
	template_name = 'apis_labels/labels_list.html'
	context_object_name = 'object_list'

	def get_queryset(self):
		return Label.objects.order_by('label')


def label_create(request):
	if request.method == "POST":
		form = LabelForm(request.POST)
		if form.is_valid():
			form.save()
			return redirect('apis_labels:label_list')
		else:
			return render (request, 'apis_labels/label_create.html', {'form':form})
	else:
		form = LabelForm()
		return render (request, 'apis_labels/label_create.html', {'form':form})


def label_edit(request, pk):
	instance = get_object_or_404(Label, pk=pk)
	if request.method == "POST":
		form = LabelForm(request.POST, instance=instance)
		if form.is_valid():
			form.save()
			return redirect('apis_labels:label_list')
		else:
			return render (request, 'apis_labels/label_create.html', {'form':form,'instance':instance})
	else:
		form = LabelForm(instance=instance)
		return render(request, 'apis_labels/label_create.html', {'form':form, 'instance':instance})


class LabelDelete(DeleteView):
	model = Label
	template_name = 'apis_templates/confirm_delete.html'
	success_url = reverse_lazy('apis_labels:label_list')



from django.conf.urls import url

from . import views, views2, detail_views, merge_views
from .autocomplete3 import (
    GenericEntitiesAutocomplete,
    GenericNetworkEntitiesAutocomplete,
)

# from .views import ReversionCompareView TODO: add again when import is fixec
from .views2 import GenericEntitiesCreateStanbolView

app_name = "apis_entities"

urlpatterns = [
    url(
        r"^entity/(?P<entity>[a-z]+)/(?P<pk>[0-9]+)/edit$",
        views2.GenericEntitiesEditView.as_view(),
        name="generic_entities_edit_view",
    ),
    url(
        r"^entity/(?P<entity>[a-z]+)/(?P<pk>[0-9]+)/detail$",
        detail_views.GenericEntitiesDetailView.as_view(),
        name="generic_entities_detail_view",
    ),
    url(
        r"^entity/(?P<entity>[a-z]+)/create$",
        views2.GenericEntitiesCreateView.as_view(),
        name="generic_entities_create_view",
    ),
    url(
        r"^entity/(?P<entity>[a-z]+)/(?P<pk>[0-9]+)/delete$",
        views2.GenericEntitiesDeleteView.as_view(),
        name="generic_entities_delete_view",
    ),
    url(
        r"^entity/(?P<entity>[a-z]+)/list/$",
        views.GenericListViewNew.as_view(),
        name="generic_entities_list",
    ),
    url(
        r"^autocomplete/createstanbol/(?P<entity>[a-zA-Z0-9-]+)/$",
        GenericEntitiesCreateStanbolView.as_view(),
        name="generic_entities_stanbol_create",
    ),
    url(
        r"^autocomplete/createstanbol/(?P<entity>[a-zA-Z0-9-]+)/(?P<ent_merge_pk>[0-9]+)/$",
        GenericEntitiesCreateStanbolView.as_view(),
        name="generic_entities_stanbol_create",
    ),
    url(
        r"^autocomplete/(?P<entity>[a-zA-Z0-9-]+)/(?P<ent_merge_pk>[0-9]+)/$",
        GenericEntitiesAutocomplete.as_view(),
        name="generic_entities_autocomplete",
    ),
    url(
        r"^autocomplete/(?P<entity>[a-zA-Z0-9-]+)/$",
        GenericEntitiesAutocomplete.as_view(),
        name="generic_entities_autocomplete",
    ),
    url(
        r"^autocomplete/(?P<entity>[a-zA-Z0-9-]+)/(?P<db_include>[a-z]+)/$",
        GenericEntitiesAutocomplete.as_view(),
        name="generic_entities_autocomplete",
    ),
    url(
        r"^autocomplete-network/(?P<entity>[a-zA-Z0-9-]+)/$",
        GenericNetworkEntitiesAutocomplete.as_view(),
        name="generic_network_entities_autocomplete",
    ),
    # TODO __sresch__ : This seems unused. Remove it once sure
    # url(r'^detail/work/(?P<pk>[0-9]+)$',
    #     detail_views.WorkDetailView.as_view(), name='work_detail'),
    url(r"^place/geojson/$", views.getGeoJson, name="getGeoJson"),
    url(r"^place/geojson/list/$", views.getGeoJsonList, name="getGeoJsonList"),
    url(r"^place/network/list/$", views.getNetJsonList, name="getNetJsonList"),
    url(
        r"^resolve/place/(?P<pk>[0-9]+)/(?P<uri>.+)$",
        views.resolve_ambigue_place,
        name="resolve_ambigue_place",
    ),
    url(r"^maps/birthdeath/$", views.birth_death_map, name="birth_death_map"),
    url(r"^networks/relation_place/$", views.pers_place_netw, name="pers_place_netw"),
    url(
        r"^networks/relation_institution/$", views.pers_inst_netw, name="pers_inst_netw"
    ),
    url(r"^networks/generic/$", views.generic_network_viz, name="generic_network_viz"),
    #    url(
    #        r'^compare/(?P<app>[a-z_]+)/(?P<kind>[a-z]+)/(?P<pk>\d+)$', ReversionCompareView.as_view()
    #    ),
    url(r"^merge-objects/$", merge_views.merge_objects, name="merge_objects"),
]



gno = 'http://www.geonames.org/ontology#'
wgs84 = "http://www.w3.org/2003/01/geo/wgs84_pos#"
gndo = 'http://d-nb.info/standards/elementset/gnd#'
owl = "http://www.w3.org/2002/07/owl#"
geo = "http://www.opengis.net/ont/geosparql#"

sett_RDF_generic = {
    'Place': {
        'data': [
            {
                'base_url': 'http://sws.geonames.org/',
                'url_appendix': 'about.rdf',
                'kind': (
                    (gno + 'featureCode', None),
                    ),
                'attributes': [
                    {
                        'name': 'name',
                        'identifiers': (
                            (('objects', 'prefName', gno + 'officialName', ('language', 'de')),),
                            (('objects', 'prefName', gno + 'officialName', ('language', 'en')),),
                            (('objects', 'prefName', gno + 'name', None),),
                            (('objects', 'prefName', gno + 'alternateName', ('language', 'de')),),
                            (('objects', 'prefName', gno + 'alternateName', ('language', 'en')),)
                        )
                    },
                    {
                        'name': 'label',
                        'identifiers': (
                            (('objects', 'label', gno + 'alternateName', ('language', 'de')),),
                            (('objects', 'label', gno + 'alternateName', ('language', 'en')),)
                        )
                    },
                    {
                        'name': 'lat',
                        'identifiers': (
                            (('objects', 'lat', wgs84 + 'lat', None),),
                        )
                    },
                    {
                        'name': 'long',
                        'identifiers': (
                            (('objects', 'lng', wgs84 + 'long', None),),
                        )
                    },
                    {
                        'name': 'parentFeature',
                        'identifiers': (
                            (('objects', 'parentFeature', gno + 'parentFeature', None),),
                        )
                    },
                    {
                        'name': 'parentCountry',
                        'identifiers': (
                            (('objects', 'parentCountry', gno + 'parentCountry', None),),
                        )
                    },
                ]
            },
            {
                'base_url': 'http://d-nb.info/gnd/',
                'url_appendix': 'about/rdf',
                'attributes': [
                    {
                        'name': 'name',
                        'identifiers': (
                            (('objects', 'prefName', gndo + 'preferredNameForThePlaceOrGeographicName', None),),
                        )
                    },
                    {
                        'name': 'label',
                        'identifiers': (
                            (
                                ('objects', 'label', gndo + 'variantNameForThePlaceOrGeographicName', None),
                            ),
                        ),
                    },
                    {
                        'name': 'latlong',
                        'identifiers': (
                            (('objects', 'latlong_base', geo + 'hasGeometry', None), '>',
                             ('objects', 'latlong_geonode', geo + 'asWKT', None)
                             ),
                        )
                    },
                ]
            }
        ],
        'matching': {
            'attributes': {
                'name': (
                    (('prefName', None),),
                ),
                'lat': (
                    (('lat', None),),
                    (('latlong_geonode', ('Point \( [+-]([0-9\.]+) [+-]([0-9\.]+)', 2)),)
                ),
                'lng': (
                    (('lng', None),),
                    (('latlong_geonode', ('Point \( [+-]([0-9\.]+) [+-]([0-9\.]+)', 1)),)
                ),
            },
            'labels': {
                'alternative name': (
                    ('label', None),
                ),
                },
            'linked objects':
            [
            {
                'type': 'Place',
                'kind': 'located in',
                'object': (
                    ('parentFeature', None),
            )
            },
            {
                'type': 'Place',
                'kind': 'located in',
                'object': (
                    ('parentCountry', None),
            )
            },
            ],
        }
    },
    'Person': {
        'data': [
            {
                'base_url': 'http://d-nb.info/gnd/',
                'url_appendix': 'about/rdf',
                'attributes': [
                    {
                        'name': 'name',
                        'identifiers': (
                            (
                             ('objects', 'prefNameNode', gndo + 'preferredNameEntityForThePerson', None),'>',
                             ('objects', 'forename', gndo + 'forename', None), '=',
                             ('objects', 'surname', gndo + 'surname', None)
                        ),
                            (
                              ('objects', 'prefNameNode', gndo + 'preferredNameEntityForThePerson', None), '>',
                                ('objects', 'descriptionNode', gndo + 'Description', None), '>',
                                ('objects', 'personalNameAddition', gndo + 'nameAddition', None), '=',
                                ('objects', 'personalNameCounting', gndo + 'counting', None), '=',
                                ('objects', 'personalName', gndo + 'personalName', None),
                            ),

                        )
                    },
                    {
                        'name': 'label',
                        'identifiers': (
                            (

                                ('objects', 'label', gndo + 'variantNameForThePerson', None),

                            ),
                        ),
                    },
                    {
                        'name': 'place of birth',
                        'identifiers': (
                        (
                            ('objects', 'place of birth', gndo + 'placeOfBirth', None),
                        ),
                    ),
                    },
                    {
                        'name': 'place of death',
                        'identifiers': (
                        (
                            ('objects', 'place of death', gndo + 'placeOfDeath', None),
                        ),
                    ),
                    },
                    {
                        'name': 'date of birth',
                        'identifiers': (
                        (
                            ('objects', 'date of birth', gndo + 'dateOfBirth', None),
                        ),
                    ),
                    },
                    {
                        'name': 'date of death',
                        'identifiers': (
                        (
                            ('objects', 'date of death', gndo + 'dateOfDeath', None),
                        ),
                    ),
                    },
                ]
            },
            {
                'base_url': 'https://www.wikidata.org/wiki/Special:EntityData/',
                'url_appendix': '.rdf',
                'attributes': [
                    {
                        'name': 'name',
                        'identifiers': (
                            (
                                ('objects', '')
                            )
                        )
                    },
                ]
            }
        ],
        'matching': {
            'attributes': {
                'name': (
                    (('surname', None),
                    ),
                    (('personalNameAddition', None),
                    )
                ),
                'first_name': (
                    (('forename', None),),
                    (('personalName', None),
                    ' ',
                     ('personalNameCounting', None)
                    ),
                ),
                'start_date_written': (
                    (('date of birth', None),),
                ),
                'end_date_written': (
                    (('date of death', None),),
                ),
            },
            'labels': {
                'alternative name': (
                    ('label', None),
                ),
            },
            'linked objects':
            [
            {
                'type': 'Place',
                'kind': 'born in',
                'object': (
                    ('place of birth', None),
            )
            },
            {
                'type': 'Place',
                'kind': 'died in',
                'object': (
                    ('place of death', None),
            )
            },
            ]
        }
    },
    'Event': {
        'data': [
            {
                'base_url': 'http://d-nb.info/gnd/',
                'url_appendix': 'about/rdf',
                'attributes': [
                    {
                        'name': 'name',
                        'identifiers': (
                            (('objects', 'prefName', gndo + 'preferredNameForTheSubjectHeading', None),),
                        )
                    },
                    {
                        'name': 'start',
                        'identifiers': (
                            (('objects', 'start_date', gndo + 'dateOfEstablishment', None),),
                        )
                    },
                    {
                        'name': 'end',
                        'identifiers': (
                            (('objects', 'end_date', gndo + 'dateOfTermination', None),),
                        )
                    },
                    {
                        'name': 'label',
                        'identifiers': (
                            (('objects', 'label', gndo + 'variantNameForTheSubjectHeading', None),),
                        )
                    },
                    {
                        'name': 'place of event',
                        'identifiers': (
                            (('objects', 'place of event', gndo + 'place', None),),
                        )
                    }
                ],
            }
        ],
        'matching': {
            'attributes': {
                'name': (
                    (('prefName', None),),
                ),
                'start_date_written': (
                    (('start_date', None),),
                ),
                'end_date_written': (
                    (('end_date', None),),
                )
            },
            'labels': {
                'alternative name': (
                    ('label', None),
                )
            },
            'linked objects': [
            {
                'type': 'Place',
                'kind': 'place of event',
                'object': (
                    ('place of event', None),
            )
            },
            ],
        }
    },
    'Institution': {
        'data': [
            {
                'base_url': 'http://d-nb.info/gnd/',
                'url_appendix': 'about/rdf',
                'attributes': [
                    {
                        'name': 'name',
                        'identifiers': (
                            (('objects', 'prefName', gndo + 'preferredNameForTheCorporateBody', None),),
                            (('objects', 'prefName', gndo + 'variantNameForTheCorporateBody', None),),
                        )
                    },
                    {
                        'name': 'alternativeName',
                        'identifiers': (
                            (('objects', 'alternativeName', gndo + 'variantNameForTheCorporateBody', None),),
                        )
                    },
                    {
                        'name': 'start',
                        'identifiers': (
                            (('objects', 'start', gndo + 'dateOfEstablishment', None),),
                        )
                    },
                    {
                        'name': 'end',
                        'identifiers': (
                            (('objects', 'end', gndo + 'dateOfTermination', None),),
                        )
                    },
                    {
                        'name': 'placeOfBusiness',
                        'identifiers': (
                            (('objects', 'placeOfBusiness', gndo + 'placeOfBusiness', None),),
                        )
                    },
                    {
                        'name': 'succeeding',
                        'identifiers': (
                            (('objects', 'succeeding', gndo + 'succeedingCorporateBody', None),),
                        )
                    },
                    {
                        'name': 'preceding',
                        'identifiers': (
                            (('objects', 'preceding', gndo + 'precedingCorporateBody', None),),
                        )
                    }
                ]
            }
        ],
        'matching': {
            'attributes': {
                'name': (
                    (('prefName', None),),
                ),
                'start_date_written': (
                    (('start', None),),
                ),
                'end_date_written': (
                    (('end', None),),
                )
            },
            'labels': {
                'alternative name': (
                    ('alternativeName', None),
                )
            },
            'linked objects': [
            {
                'type': 'Place',
                'kind': 'located in',
                'object': (
                    ('placeOfBusiness', None),
            )
            },
            {
                'type': 'Institution',
                'kind': 'preceding',
                'object': (
                    ('preceding', None),
            )
            },
            {
                'type': 'Institution',
                'kind': 'succeeding',
                'object': (
                    ('succeeding', None),
            )
            },
            ]
        }
    }
}



# Generated by Django 2.1.2 on 2020-02-16 11:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_entities', '0002_auto_20200121_1227'),
    ]

    operations = [
        migrations.AlterField(
            model_name='person',
            name='gender',
            field=models.CharField(blank=True, choices=[('female', 'female'), ('male', 'male'), ('third gender', 'third gender')], max_length=15),
        ),
    ]



# Generated by Django 2.1.12 on 2020-01-21 12:27

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Label',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(blank=True, help_text='The entities label or name.', max_length=255)),
                ('isoCode_639_3', models.CharField(blank=True, default='deu', help_text="The ISO 639-3 (or 2) code for the label's language.", max_length=3, null=True, verbose_name='ISO Code')),
            ],
        ),
    ]
